#include <QDir>
#include <QDebug>
#include <QDateTime>

#include "cleanser.h"
#include "utils.h"
#include "logger.h"

Cleanser::Cleanser(Settings *settings, QObject *parent) :
    QObject(parent)
{
    this->settings = settings;
    m_etagDir.setPath(QDir::homePath() + "/.linutop-kiosk/etag");
    m_cacheDir.setPath(QDir::homePath() + "/.linutop-kiosk/data");
}

void Cleanser::clean()
{
    Q_FOREACH (QString file, m_toClean) {
        if ((file == ".") || (file == ".."))
            continue;
        qDebug() << "Deleting" << file;
        QFile::remove(m_etagDir.path() + "/" + file);
        QFile::remove(m_cacheDir.path() + "/" + file);
    }
}

void Cleanser::cleanUntilFree(qlonglong available, qint64 space_needed)
{
    QDateTime oneDayAgo = QDateTime::currentDateTime().addDays(-1);
    Q_FOREACH (QString file, m_toClean) {
        if ((file == ".") || (file == ".."))
            continue;
        QFileInfo fileInfo(file);
        if(fileInfo.created() < oneDayAgo)
            continue;
        if (space_needed < available)
        {
            Q_EMIT ok();
            return;
        }
        qDebug() << "Deleting" << file;
        QFile::remove(m_etagDir.path() + "/" + file);
        QFile::remove(m_cacheDir.path() + "/" + file);
    }
    Q_EMIT notOk();
}

void Cleanser::checkThat(qint64 space_requiered)
{
    qlonglong available = Utils::availableDiskSpace();
    //qDebug() << space_requiered << available;
    if(available < space_requiered)
    {
        qDebug() << "Size cleansing";
        m_toClean = m_etagDir.entryList();
    }
    else{
        Q_EMIT ok();
        return;
    }
    cleanUntilFree(available, space_requiered);
}

void Cleanser::check()
{
    qlonglong available = Utils::availableDiskSpace();
    //qlonglong total = Utils::totalDiskSpace();
    if (available < 50000000) //cache size limit 50MB
    {
        qDebug() << "Size cleansing";
        m_toClean = m_etagDir.entryList();
    }
    clean();
}
