#ifndef DISPATCHER_H
#define DISPATCHER_H

#include <QString>
#include <QUrl>
#include <QProcess>

#include "playlistparser.h"
#include "playlistloop.h"
#include "runnable.h"
#include "ui/viewer.h"
#include "remote.h"
#include "settings.h"
#include "cleanser.h"


class CommandLineParser;
class Dispatcher : public QObject
{
    Q_OBJECT

public:
    Dispatcher(Viewer *viewer, CommandLineParser const* cmdparse);
    ~Dispatcher(void);

    void startRemote();

public Q_SLOTS:

    bool next(void);
    void shootScreen(void);
    void finalize(void);
    void endIdle(void);
    void startInlineRemote(void);
    bool isValid(void) const { return m_is_valid; }

private:
    enum Status {
        StatusNone = 0,
        StatusBrowser,
        StatusAfterBrowser,
        StatusPlaylist
    };

    Settings *settings;
    Cleanser *cleanser;
    Dispatcher::Status m_status;
    Runnable *m_currentRunnable;
    QWidget* m_currentWidget;
    QString m_mode;
    Viewer *m_viewer;
    QDateTime m_lastUpdate;
    Remote* remote;
    bool m_plRefresh;
    QTimer m_plTimer;
    QFile m_file;
    bool m_updateMode;

    PlaylistParser *m_pl;
    PlaylistParser *m_ppl;
    PlaylistLoop *m_loop;
    int m_loopIdx;
    QList<int> m_loopIdxList;
    int m_listIdx;

    bool m_idling;
    bool m_is_valid;
    bool m_oldPl;

    QString remotePid;

    QTimer m_ssTimer;

    QDateTime start_sec;
private:
    void disableScreensaver();

    bool startup(void);
    void reconnectMouseEvent();
    void afterBrowser();
    bool startBrowser(QUrl url = QUrl());
    void execHook(QString hook);
    QDateTime getLastEdit();
    bool view_next();

private Q_SLOTS:
    void onReconnectMouseEventTimeout();
    void onScreensaverTimeout();
    void onMouseEvent();
    void onLeftClick();
    void showLoopNext();
    void onPlTimeout();
    void startSlideshow();
    void goTo(int index, int loop);
    void goTo(QString);
    void startNoScreensaver(int, QProcess::ExitStatus);
};

#endif
