#ifndef DOWNLOADER_H
#define DOWNLOADER_H

#include <QNetworkConfigurationManager>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QString>
#include <QObject>
#include <QList>
#include <QFile>
#include <QFileInfo>
#include <QSettings>

#include "settings.h"
#include "playlistitem.h"
#include "playlistloop.h"
#include "cleanser.h"

class ItemDownloader : public QObject
{
    Q_OBJECT

public:
    ItemDownloader(QString &uri, QString &dest, QString &etag, bool &mode);
    ~ItemDownloader();
    void start();

private:
    QString m_baseUrl;
    QString m_uri;
    QFile m_file;
    QFile m_etagFile;
    QNetworkRequest m_request;
    QNetworkReply *m_reply;
    QNetworkAccessManager *m_nam;
    bool m_update;
    bool m_coLost;
    qint64 m_bytesRec;
    qint64 m_bytesTotal;
    Settings *m_settings;
    Cleanser *m_cleanser;


    bool inCache();
    void pauseWhileCheckingSize();

private Q_SLOTS:
    void onReadyRead();
    void onFinished();
    void checkSizeFileFirst(qint64 bytesRec, qint64 bytesTotal);
    void namChanged(QNetworkAccessManager::NetworkAccessibility nam);
    void resumeDownload();
    void notEnoughSpace();

Q_SIGNALS:
    void onError();
    void sizeIssue();
    void finished();
};


class Downloader : public QObject
{
    Q_OBJECT

public:
    Downloader(PlaylistLoop *loop, bool mode);
    ~Downloader();
    void start();
    bool isDone(){return m_done;}
    void setCountError(int);

private:
    PlaylistLoop *m_loop;
    QFile m_file;
    QString m_baseUrl;
    QString m_cacheDir;
    QString m_dataDir;
    QString m_etagDir;
    bool m_done;
    bool m_updateMode;
    ItemDownloader *m_idl;
    int m_index;
    int m_countError;



private Q_SLOTS:
    void downloadNext();
    void addError();
    void replaceFile();

Q_SIGNALS:
    void downloadDone();
    void downloadNotDone();
};

#endif
