#include <QApplication>
#include <QNetworkProxyFactory>
#include <QIcon>
#include <QDir>
#include <QTranslator>

    /*
#ifdef Q_WS_X11
    #include <X11/Xlib.h>
#endif
    */

#include "dispatcher.h"
#include "ui/viewer.h"
#include "proxyfactory.h"
#include "settings.h"
#include "logger.h"
#include "utils.h"
#include "config.h"
#include "commandlineparser.h"

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

// Define the function to be called when ctrl-c (SIGINT) signal is sent to process
void
signal_callback_handler(int signum)
{
    qDebug() << "Caught signal" << signum;

    qApp->exit(signum);
}


int main(int argc, char **argv)
{
    // Register signal and signal handler
    signal(SIGINT, signal_callback_handler);

//#ifdef Q_WS_X11
//        XInitThreads();
//#endif

    // Create config directory if it doesn't exist.
    QDir config_dir(qgetenv("HOME") + "/.linutop-kiosk");
    if (!config_dir.exists()) config_dir.mkdir(config_dir.path());

    CommandLineParser cmdparse(argc, argv);
    if (cmdparse.needHelp())
    {
        cmdparse.displayHelp();
        return 0;
    }
    QApplication app(argc, argv);
    app.setApplicationName("linutop-kiosk");
    app.setOrganizationName("Linutop");
    app.setOrganizationDomain("linutop.com");

    app.quitOnLastWindowClosed();

    QTranslator qtTranslator;
    qtTranslator.load("browser_" + QLocale::system().name(),
                      "/usr/share/linutop-webkiosk/i18n");
    app.installTranslator(&qtTranslator);

    Logger::init();

    QIcon::setThemeName(Utils::getIconTheme());

    ProxyFactory *factory = new ProxyFactory(&cmdparse);
    QNetworkProxyFactory::setApplicationProxyFactory(factory);

    Viewer v;

    QProcess p;
    p.start("xset", QStringList() <<"dpms"<< "0"<<"0"<<"0");

    Dispatcher d(&v, &cmdparse);
    if (!d.isValid()) return 1;

    int r = 0;
    if (d.next()) r = app.exec();
    Logger::shutdown();

    return r;
}
