#ifndef PLAYLISTITEM_H
#define PLAYLISTITEM_H

#include <QString>
#include <QMap>
#include <QDomNode>

class PlaylistItem
{
public:
    enum Type {
        TypeUndefined = -1,
        TypeRemote = 0,
        TypeVideo,
        TypeGif,
        TypeImage,
        TypePdf,
        TypeUrl,
        TypeWebpage,
        TypeDocument,
        TypeDir,
        TypeIdle,
        TypeMaintenance,
        TypeNotSupported
    };

    PlaylistItem(QString uri, PlaylistItem::Type type, int time, bool href = false, QStringList param = QStringList());
    bool isEmpty() const { return m_uri.isEmpty(); }

    PlaylistItem::Type type() const { return m_type; }
    void setTime(unsigned int time) { m_time = time; }
    QString uri() const { return m_uri; }
    void setUri(QString &uri) { m_uri = uri; }
    int time() const { return m_time; }
    bool href() const { return m_href; }
    QStringList params() const { return m_params; }

    static PlaylistItem * idle();
    static PlaylistItem *fromXmlNode(QDomNode *node);
    static QString typeToStr(PlaylistItem::Type type);

private:
    QString m_uri;
    PlaylistItem::Type m_type;
    int m_time;
    bool m_href;
    QStringList m_params;

    static QMap<PlaylistItem::Type, QString> initStrings();
    static QMap<PlaylistItem::Type, QString> typeToStrMap;
};

#endif
