#ifndef PLAYLISTLOOP_H
#define PLAYLISTLOOP_H

#include <QDateTime>
#include <QDate>
#include <QTime>
#include <QBitArray>

#include "playlistitem.h"

class PlaylistLoop
{
public:
    PlaylistLoop(int default_time, QDomNode &node, QString baseUrl);
    PlaylistLoop(int default_time, QString &data, QString baseUrl);
    ~PlaylistLoop();


    int count() const { return m_items.count(); }
    void refresh();
    PlaylistItem * at(int i) { return m_items.at(i); }
    bool isEmpty() const { return m_id.isEmpty(); }
    bool downloadable() const { return m_downloadable; }
    QString id() const { return m_id; }
    int priority() const { return m_priority; }
    QDateTime nowOk(QDateTime start_sec);
    QString baseUrl() const { return m_baseUrl; }
    QList<PlaylistItem *> const& items() const { return m_items; }
    QList<PlaylistItem *>& items(void) { return m_items; }
    QDateTime getFromIdle(QString uri);
    QDateTime getToIdle(QString uri);


private:
    QString m_baseUrl;
    QString m_id;
    bool m_idle;
    QString m_startTimeStr;
    QString m_endTimeStr;
    QDateTime m_startTime;
    QDateTime m_endTime;
    QString m_weekdaysStr;
    QBitArray m_weekdays;
    QDateTime m_startDate;
    QDateTime m_endDate;
    QString m_startDateStr;
    QString m_endDateStr;
    int m_priority;
    bool m_downloadable;
    int m_default_time;
    bool m_shuffle;
    QDateTime from;
    QDateTime to;

    QList<PlaylistItem *> m_items;

    int m_curIdx;

    // For local playlist.
    void fromString(QString &data);

    // For server playlist.
    void fromXmlNode(QDomNode &node);
    void childFromXmlNode(QDomNode &node);

    bool addDirectoryInItems(QString const& path, int time);

    bool validate();
    bool validateDate();
    bool validateTime();
    bool validateWeekdays();

    void addDemoItems();
};

#endif
