#include <QDomDocument>
#include <QApplication>
#include <QDir>
#include <QFile>
#include <QTextStream>
#include <QDebug>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QEventLoop>
#include <QMessageBox>

#include "playlistparser.h"

FlatPlaylistParser::FlatPlaylistParser(
        int default_time, QString &data)
    : PlaylistParser(default_time)
    , m_loop(0)
    , m_data(data)
{ m_loop = new PlaylistLoop(defaultTime(), data, QString()); }

PlaylistLoop *FlatPlaylistParser::selectLoop(QDateTime)
{ return m_loop; }

void FlatPlaylistParser::refresh()
{
    if (m_loop) delete m_loop;

    m_loop = new PlaylistLoop(defaultTime(), m_data, QString());
}

/* */
XmlPlaylistParser::XmlPlaylistParser(
        int default_time, QString &data, QString baseUrl)
    : PlaylistParser(default_time)
    , m_baseUrl(baseUrl)
    , m_valid(false)
{
    QDomDocument xmldoc;
    if (!xmldoc.setContent(data))
        return;

    m_valid = true;
    m_name = xmldoc.elementsByTagName("schedule").at(0)
            .attributes().item(0).nodeValue();

    QDomNodeList loops = xmldoc.elementsByTagName("loop");
    for (int i = 0; i < loops.count(); ++i)
    {
        QDomNode loop = loops.at(i);
        m_loops << new PlaylistLoop(defaultTime(), loop, m_baseUrl);
    }
}

void XmlPlaylistParser::refresh()
{
}

PlaylistLoop *XmlPlaylistParser::selectLoop(QDateTime start_sec)
{
    PlaylistLoop *selected = m_loops.at(0);
    QDateTime maxDate, date, emptyDate;
    for (int i = 1; i < m_loops.count(); i++)
    {
        PlaylistLoop *loop = m_loops.at(i);
        date = loop->nowOk(start_sec);
        if(date != emptyDate)
        {
            if(date > maxDate)
            {
                maxDate = date;
                selected = m_loops.at(i);
            }
        }
    }
    return selected;
}

PlaylistParser *PlaylistParser::get(
        int default_time, QUrl url, QWidget* to_hide_on_error)
{
    QString data;
    if (url.scheme() == "file")
    {
        QFile datafile(url.path());
        datafile.open(QIODevice::ReadOnly | QIODevice::Text);
        QTextStream in(&datafile);
        data = in.readAll();
        datafile.close();
    }
    else
    {
        QNetworkRequest req(url.toString());
        QNetworkAccessManager nam;
        QEventLoop loop;
        QNetworkReply *reply = nam.get(req);
        connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
        loop.exec();
        loop.deleteLater();

        QFile f(QDir::homePath() + "/.linutop-kiosk/playlist");
        QFile newpl(QDir::homePath() + "/.linutop-kiosk/playlist-old");

        QString oldData;
        /*if(reply->readAll() == "Unavailable for public use")
        {
            to_hide_on_error->hide();
            QMessageBox messageBox;
            messageBox.critical(
                        0, "Unavailable for public use",
                        "You try to read a private channel.\n"
                        "Please select another channel or make it public.");
            messageBox.setFixedSize(500,200);
            qApp->quit();
        }*/
        if(f.exists() && f.open(QIODevice::ReadOnly))
        {
            oldData = f.readAll();
            if(!newpl.exists())
            {
                if(newpl.open(QIODevice::WriteOnly))
                {
                    newpl.write(oldData.toUtf8());
                    newpl.close();
                }
            }
            f.close();
        }

        if (reply->error() != QNetworkReply::NoError)
        {
            if (!f.exists())
            {
                to_hide_on_error->hide();
                QMessageBox messageBox;
                messageBox.critical(
                            0, "Network unreachable",
                            "Linutop Kiosk is Offline.\n"
                            "Please select local playlist option or connect "
                            "the system to a network.");
                messageBox.setFixedSize(500,200);
                qApp->quit();
            }

            // on error, try to use a cached playlist
            if (newpl.open(QIODevice::ReadOnly))
            {
                data = newpl.readAll();
                newpl.close();
            }
        }
        else
        {
            data = reply->readAll();
            if(data != "")
            {
                // save the playlist
                if (f.open(QIODevice::WriteOnly))
                {
                    if(data == "Unavailable for public use")
                    {
                        data = "<?xml version=\"1.0\"?>\n"
                                "<schedule>\n"
                                "<loop id=\"0\" idle=\"0\">\n"
                                "</loop>\n"
                                "</schedule>";
                        f.write(data.toUtf8());
                        f.close();
                    }
                    else
                    {
                        f.write(data.toUtf8());
                        f.close();
                    }
                }
            }
            else
            {
                // on error, try to use a cached playlist
                if (newpl.open(QIODevice::ReadOnly))
                {
                    data = newpl.readAll();
                    newpl.close();
                }
            }
        }
    }

    XmlPlaylistParser *xmlp = new XmlPlaylistParser(
                default_time, data, url.toString());
    if (xmlp->isValid())
        return xmlp;

    delete xmlp;

    FlatPlaylistParser *flatp = new FlatPlaylistParser(
                default_time, data);
    return flatp;
}
