#ifndef PLAYLISTPARSER_H
#define PLAYLISTPARSER_H

#include <QString>
#include <QUrl>
#include <QDateTime>

#include "playlistloop.h"

class PlaylistParser : public QObject
{
    Q_OBJECT

public:
    PlaylistParser(int default_time)
        : QObject(), m_default_time(default_time) {}
    static PlaylistParser *get(int default_time, QUrl url, QWidget *to_hide_on_error = NULL);
    virtual PlaylistLoop *selectLoop(QDateTime) = 0;
    virtual void refresh() = 0;
    virtual QList<PlaylistLoop *> getLoops() = 0;

    int defaultTime(void) const { return m_default_time; }

private:
    int m_default_time;
};

class FlatPlaylistParser : public PlaylistParser
{
public:
    FlatPlaylistParser(int default_time, QString &data);
    PlaylistLoop *selectLoop(QDateTime);
    void refresh();
    QList<PlaylistLoop *> getLoops(){ return QList<PlaylistLoop *>()<<m_loop;}

private:
    PlaylistLoop *m_loop;
    QString m_data;
};

class XmlPlaylistParser : public PlaylistParser
{
public:
    XmlPlaylistParser(
            int default_time, QString &data, QString baseUrl = QString());
    bool isValid() { return m_valid; }
    PlaylistLoop *selectLoop(QDateTime start_sec);
    void refresh();
    QString getName(){ return m_name;}
    QList<PlaylistLoop *> getLoops(){ return m_loops;}

private:
    QString m_baseUrl;
    QList<PlaylistLoop *> m_loops;
    bool m_valid;
    QString m_name;
};

#endif
