#include <QFile>
#include <QSettings>
#include <QTextStream>

#include <QDebug>
#include <QMimeDatabase>

#include "processmanager.h"

static QString cmdForMime(QString mime);
static QString cmdForFile(QString filePath);

RProcess::RProcess(QString cmd, QString filePath, QObject *parent)
    : QProcess(parent)
    , m_cmd(cmd)
    , m_filePath(filePath)
{
    connect(this, SIGNAL(finished(int, QProcess::ExitStatus)),
            this, SLOT(onFinished(int, QProcess::ExitStatus)));
}

void RProcess::go()
{
    start(m_cmd);
}

void RProcess::onFinished(int /*exitCode*/, QProcess::ExitStatus /*exitStatus*/)
{
    QFile f(m_filePath);
    f.remove();
}


void SubprocessManager::empty()
{
    Q_FOREACH (const RProcess *process, m_processes)
        delete process;

    m_processes.clear();
}

bool SubprocessManager::open(QString filePath, QString mime)
{
    QString cmd;
    QMimeDatabase qmdb;
    QMimeType mimetype = qmdb.mimeTypeForName(mime);
    if (! mimetype.isDefault())
        cmd = cmdForMime(mimetype.name());
    else
        cmd = cmdForFile(filePath);

    if (cmd.isEmpty())
        return false;

    QString escaped_path = QString ("\"%1\"").arg(filePath);
    if (! cmd.contains(QChar('%'))) {
        cmd = cmd + " " + escaped_path;
    } else {
        cmd = cmd.replace("%u", escaped_path).replace("%U", escaped_path);
        cmd = cmd.replace("%f", escaped_path).replace("%F", escaped_path);
    }

    RProcess *p = new RProcess(cmd, filePath, this);
    p->go();
    m_processes << p;

    return true;
}

static QString cmdForMime(QString mime)
{
    QStringList l;

    QFile cache("/usr/share/applications/mimeinfo.cache");
    if (cache.open(QIODevice::ReadOnly))
    {
        QTextStream in(&cache);
        while (!in.atEnd())
        {
            QString line = in.readLine();
            if (line.startsWith(mime + "="))
            {
                QStringList tmp = line.split("=");
                l = tmp[1].split(";");
                break;
            }
        }
    }
    cache.close();

    QString app;
    Q_FOREACH (const QString &desktopFile, l)
    {
        QFile f("/usr/share/applications/" + desktopFile);
        if (!f.exists())
            continue;

        QSettings s(f.fileName(), QSettings::IniFormat);
        app = s.value("Desktop Entry/Exec").toString();
        break;
    }

    return app;
}

static QString cmdForFile(QString filePath)
{
    QString mimeGuessed;
    QMimeDatabase db;
    QMimeType mime = db.mimeTypeForFile(filePath);
    if (mime.name().isEmpty()) {
        QString cmd = QString("file -b --mime-type \"%1\"").arg(filePath);
        QProcess p;
        p.start(cmd);
        p.waitForFinished(-1);
        mimeGuessed = p.readAllStandardOutput().simplified();
    }

    if (mimeGuessed.isEmpty())
        return QString();
    else
        return cmdForMime(mimeGuessed);
}
