#include "remote.h"

#include <QScriptEngine>
#include <QFile>


Remote::Remote(void)
{
    qDebug()<<"Remote handler created.";
    this->bind(QHostAddress::LocalHost, 54545);
    connect(this, SIGNAL(readyRead()), this, SLOT(processMessage()));
}


enum action_t
{
    PAUSE            = 0,
    NEXT             = 1,
    GOTO             = 2,
    SCREENSHOT       = 3,
    RESTART          = 4,
    TEXT             = 5,
    LOOP             = 6,
    URL              = 7,
    DELETE_DRAG_DROP = 8
};

void Remote::processMessage()
{
    while (hasPendingDatagrams())
    {
        QByteArray datagram;
        datagram.resize(pendingDatagramSize());
        QHostAddress sender;
        quint16 senderPort;

        readDatagram(datagram.data(), datagram.size(),
                     &sender, &senderPort);
        QString message = QString(datagram);
        qDebug() << "Received DataGram:" << message.toStdString().c_str();
        QScriptEngine engine;
        QScriptValue v = engine.evaluate("("+message+")");
        switch(v.property("action").toVariant().toInt())
        {
            case PAUSE:
                Q_EMIT pause();
                break;
            case GOTO:
                this->processGoTo(&v);
                break;
            case NEXT:
                Q_EMIT next();
                break;
            case RESTART:
                Q_EMIT restart();
                break;
            case SCREENSHOT:
                Q_EMIT screenshot();
                break;
            case TEXT:
            {
                QString txt = v.property("data").property("text").toString();
                int duration = v.property("data").property("duration")
                        .toVariant().toInt();
                if (txt != "") Q_EMIT text(txt, duration);
                //Q_EMIT screenshot();
                break;
            }
            case LOOP:
                Q_EMIT loop();
                break;
            case URL:
            {
                QString url = v.property("data").property("url")
                        .toVariant().toString();
                int duration = v.property("data").property("duration")
                        .toVariant().toInt();
                Q_EMIT readUrl(url, duration);
                break;
            }
            case DELETE_DRAG_DROP:
            {
                QString path = v.property("data").property("path")
                        .toVariant().toString();
                QString filepath = qgetenv("HOME") +
                        "/.linutop-kiosk/dragndrop/" + path;
                if (QFile::exists(filepath))
                {
                    QFile::remove(filepath);
                    qDebug() << "Remove drag and drop file:" << path;
                    Q_EMIT restart();
                }
                else
                {
                    qDebug() << "Drag and drop file not removed "
                                "because doesn't exists:" << path;
                }
                break;
            }
        }
    }
}


void Remote::processGoTo(QScriptValue* js)
{
    QScriptValue data = js->property("data");
    if (data.property("filePath").isValid())
        Q_EMIT goTo(QString(data.property("filePath").toString()));
    else
        Q_EMIT goTo(data.property("index").toInteger(),
                  data.property("loop").toInteger());
}
