#include "system/panel.h"

#include <QProcess>


namespace panel {

bool Manager::is_fullscreen() const
{ return m_fullscreen; }

bool Manager::use_password() const
{ return m_use_password; }

Manager::Manager(bool fullscreen, bool use_password)
    : m_fullscreen(fullscreen)
    , m_use_password(use_password) {}

/**
 * @brief The LXDE panel management.
 */
class LXDE : public Manager
{
public:

    LXDE(bool fullscreen, bool password)
        : Manager(fullscreen, password)
    {
        if (is_fullscreen() && use_password())
        {
            // Check if the toolbar is active.
            {
                QProcess lxde;
                lxde.start("pgrep", QStringList() << "lxpanel");
                lxde.waitForFinished();
                QByteArray result = lxde.readAllStandardOutput();

                // If panel is not active => nothing to do.
                if (result.size() == 0) return;
            }

            // Remove the toolbar.
            {
                QProcess lxde;
                lxde.startDetached("pkill", QStringList() << "-9" << "lxpanel");
            }
        }
    }

    virtual ~LXDE(void)
    {
        if (is_fullscreen() && use_password())
        {
            // Recreate the toolbar.
            {
                QProcess lxde;
                lxde.startDetached("lxpanel", QStringList() << "--profile"
                   #ifdef __arm__
                           << "LXDE-pi"
                   #else
                           << "LXDE"
                   #endif
                           );
            }
        }
    }
};

/**
 * @brief The XFCE panel management.
 */
class XFCE : public Manager
{
public:

    XFCE(bool fullscreen, bool password)
        : Manager(fullscreen, password)
    {
        if (is_fullscreen() && use_password())
        {
            // Check if the toolbar is active.
            {
                QProcess xfce;
                xfce.start("pgrep", QStringList() << "xfce4-panel");
                xfce.waitForFinished();
                QByteArray result = xfce.readAllStandardOutput();

                // If panel is not active => nothing to do.
                if (result.size() == 0) return;
            }

            // Remove the toolbar.
            {
                QProcess xfce;
                xfce.startDetached("xfce4-panel", QStringList() << "-q");
            }
        }
    }

    virtual ~XFCE(void)
    {
        if (is_fullscreen() && use_password())
        {
            // Recreate the toolbar.
            {
                QProcess xfce;
                xfce.startDetached("xfce4-panel");
            }
        }
    }
};

Manager *Manager::create(bool fullscreen, bool use_password)
{
    // Test if LXDE.
    {
        QProcess lxde;
        lxde.start("pgrep", QStringList() << "lxsession");
        lxde.waitForFinished();
        QByteArray result = lxde.readAllStandardOutput();
        if (result.size() != 0) return new LXDE(fullscreen, use_password);
    }

    // Test if XFCE.
    {
        QProcess xfce;
        xfce.start("pgrep", QStringList() << "xfce4-session");
        xfce.waitForFinished();
        QByteArray result = xfce.readAllStandardOutput();
        if (result.size() != 0) return new XFCE(fullscreen, use_password);
    }

    // Otherwise.
    {
        return new Manager(fullscreen, use_password);
    }
}

} // panel
