#include "usbauto.h"

#include <cstdlib>
#include <unistd.h>

#include <QDBusInterface>
#include <QDBusArgument>
#include <QDebug>
#include <QDirIterator>
#include <QProcess>
#include <QMessageBox>
#include <QTimer>
#include <QThread>

#include "windowmanagerkiller.h"


UsbAuto::UsbAuto(const QString &remoteKey)
    : m_key(remoteKey)
    , m_active(false)
    , m_killer(WindowManagerKiller::create())
{
    if (!QDBusConnection::sessionBus().connect(
        #if defined(__arm__) || !defined(__x86_64__)
                QString("org.gtk.Private.UDisks2VolumeMonitor"),
        #else
                QString("org.gtk.vfs.UDisks2VolumeMonitor"),
        #endif
                QString(""),
                QString("org.gtk.Private.RemoteVolumeMonitor"),
                QString("MountAdded"), this,
                SLOT(onUsbKeyMounted(QDBusMessage))))
    {
        qDebug() << "Unable to handle usb key connections: "
                 << QDBusConnection::sessionBus().lastError().message();
    }

    if (!QDBusConnection::sessionBus().connect(
        #if defined(__arm__) || !defined(__x86_64__)
                QString("org.gtk.Private.UDisks2VolumeMonitor"),
        #else
                QString("org.gtk.vfs.UDisks2VolumeMonitor"), //changement vfs v64 bits
        #endif
                QString(""),
                QString("org.gtk.Private.RemoteVolumeMonitor"),
                QString("VolumeAdded"), this,
                SLOT(onUsbKeyPlugged(QDBusMessage))))
    {

        qDebug() << "Unable to handle usb key connections: "
                 << QDBusConnection::sessionBus().lastError().message();
    }

    qDebug() << "UsbAuto handler created...";
}


UsbAuto::~UsbAuto(void)
{ qDebug() << "UsbAuto handler destroyed..."; }


static bool mount(QString const& device, QString const& uuid)
{
    QProcess mounter;
    QString cmd;
    QStringList args;
#ifdef DEBUG
    cmd = "sudo";
    args << LINUTOP_USB_UPDATE;
#else
    cmd = "sudo";
#endif // DEBUG
    QString uid = QString::number(getuid());
    args << "linutop-usbupdate" <<  QString(qgetenv("USER"))
          << uid << uuid << device << "mount";
    qDebug() << "UID:" << uid;
    qDebug() << "Running: " << cmd.toStdString().c_str()
             << args.join(" ").toStdString().c_str();

    mounter.start(cmd, args);
    return mounter.waitForFinished();
}


static bool umount(QString const& device, QString const& uuid)
{
    QProcess umounter;
    QString cmd;
    QStringList args;
#ifdef DEBUG
    cmd = "sudo";
    args << LINUTOP_USB_UPDATE;
#else
    cmd = "sudo";
#endif // DEBUG
    QString uid = QString::number(getuid());
    args << "linutop-usbupdate" << QString(qgetenv("USER"))
          << uid << uuid << device << "umount";
    qDebug() << "UID:" << uid;
    qDebug() << "Running: " << cmd.toStdString().c_str()
             << args.join(" ").toStdString().c_str();

    umounter.start(cmd, args);
    return umounter.waitForFinished();
}


void UsbAuto::onUsbKeyMounted(QDBusMessage message)
{
    if (message.arguments().size() < 3 ||
            !message.arguments().at(2).canConvert<QDBusArgument>())
        return;

    QDBusArgument const& arg =
            message.arguments().at(2).value<QDBusArgument>();

    // Check if message is well formed.
    if (arg.currentType() != QDBusArgument::StructureType ||
            !arg.currentSignature().contains(QString("(sssss")))
    {
        qDebug() << "DBUS Message is not well formed:"
                 << arg.currentSignature();
        return;
    }

    QString path;
    arg.beginStructure();
    {
        QString address, name, icon1, icon2, empty;
        arg >> address >> name >> icon1 >> icon2 >> empty >> path;
        qDebug() << path;
    }
    arg.endStructure();

    path.replace(QRegExp("^file://"), "");
    QDir mountPoint(path);
    qDebug() << (mountPoint.exists()
                 ? ("USB key mounted: " + path)
                 : ("USB key mount failed: " + path));

    if (mountPoint.exists())
    {
        QMessageBox* message = new QMessageBox;
        QObject::connect(this, SIGNAL(finished()), message, SLOT(deleteLater()));
        QTimer::singleShot(3000, this, SLOT(copyFiles()));
        message->setText(tr("Please wait while copying files from USB."));
        message->setStandardButtons(NULL);
        message->setStyleSheet("QLabel{min-width:500 px; font-size: 24px;}");
        message->setWindowTitle(tr("Linutop auto USB Update"));
        message->exec();
    }
}


void UsbAuto::onUsbKeyPlugged(QDBusMessage message)
{

    QProcess k;
    k.start("killall", QStringList() << "Thunar");
    k.waitForFinished();
    qDebug() << "Thunar killed.";

    //Supprimer le repertoire usbauto

    QProcess supprimer;
    supprimer.start("sh", QStringList()<<"-c"<< "rm -r /home/linutop/.linutop-kiosk/usbauto");
    supprimer.waitForFinished();
    qDebug()<<"sudo rm -r /home/linutop/.linutop-kiosk/usbauto";

    // Check arguments.
    if (message.arguments().size() < 3 ||
            !message.arguments().at(2).canConvert<QDBusArgument>())
        return;

    QDBusArgument const& arg =
            message.arguments().at(2).value<QDBusArgument>();

    // Check if message is well formed.
    if (arg.currentType() != QDBusArgument::StructureType ||
            !arg.currentSignature().contains(QString("(ssssssbbssa")))
    {
        qDebug() << "DBUS QLabel{min-width: 500px;}Message is not well formed:"
                 << arg.currentSignature();
        return;
    }

    arg.beginStructure();
    {
        QString adrs1, adrs2, name, icon1, icon2, empty1, empty2, empty3;
        bool b1, b2;
        QMap<QString, QString> map;

        arg >> adrs1 >> name >> icon1
            >> icon2
            >> empty1 >> empty2
            >> b1 >> b2
            >> adrs2 >> empty3
            >> map;
        if (!map.count("unix-device") || !map.count("uuid"))
        {
            qDebug() << "Unable to get unix-device name.";
            return;
        }
        QString dev = map["unix-device"], uuid = map["uuid"];
        qDebug() << "Plugged device:" << dev;
        if (mount(dev, uuid))
        {
            m_device = dev;
            m_uuid = uuid;
            m_active = true;
        }
        else qDebug() << "Unable to mount:" << dev;
    }

    arg.endStructure();

}


static bool removeDirectory(QDir const& dir)
{
    bool result = true;

    if (dir.exists())
    {
        Q_FOREACH(QFileInfo info, dir.entryInfoList(
                    QDir::NoDotAndDotDot | QDir::System | QDir::Hidden |
                    QDir::AllDirs | QDir::Files, QDir::DirsFirst))
        {
            if (info.isDir())
                result = removeDirectory(info.absoluteFilePath());
            else result = QFile::remove(info.absoluteFilePath());

            if (!result) return result;

        }
        result = dir.rmdir(dir.path());
    }

    return result;
}


void UsbAuto::copyFiles(void)
{
    if (!m_active) return;


    qDebug() << "Enter copy";

    QString user = qgetenv("USER");
    QString path = "/media/" + user + "/" + m_uuid + "/" + m_key;

    QDir mounted(path);
    if (mounted.exists())
    {
        // Clear directory.
        QDir config_dir(qgetenv("HOME") + "/.linutop-kiosk");
        if (!config_dir.exists()) config_dir.mkdir(config_dir.path());

        QDir copy_dir(qgetenv("HOME") + "/.linutop-kiosk/usbauto");
        removeDirectory(copy_dir);
        copy_dir.mkdir(copy_dir.path());

        // Copy it again.

    #if defined(__arm__) || !defined(__x86_64__)
        QStringList qsl =
               mounted.entryList(
                 QDir::NoDotAndDotDot | QDir::Dirs | QDir::Files);
        Q_FOREACH (QString file, qsl)
        {
           QFileInfo finfo(QString("%1/%2")
                          .arg(mounted.path()).arg(file));
        if (finfo.isSymLink()) continue;
        if (finfo.isDir()) continue;

        if (finfo.completeSuffix().contains("ppt") || finfo.completeSuffix().contains("doc") || finfo.completeSuffix().contains("xls")) {
            qDebug() << "ppt/doc/xls files are not accepted.";
            continue;
        }

            QString to_copy = QDir::toNativeSeparators(finfo.filePath());
            qDebug() << "Copying:" << to_copy;

            QFile::copy(to_copy, QString("%1/%2")
                             .arg(copy_dir.path()).arg(file));
       }
    #else
        QProcess copier;
        copier.start("sh", QStringList()<<"-c"<< "cp " + path + "*" +  " /home/linutop/.linutop-kiosk/usbauto");
        copier.waitForFinished();
        qDebug()<<"cp " + path + "*" +  " /home/linutop/.linutop-kiosk/usbauto";

        // Fix bug coding
        QProcess accent;
        accent.start("sh", QStringList()<<"-c"<< "cd /home/linutop/.linutop-kiosk/usbauto && find . \\! -name \\* -exec convmv -f iso-8859-15 -t utf8 -r --nosmart --notest '{}' \\;");
        accent.waitForFinished();

        QProcess umountall;
        umountall.start("sh", QStringList()<<"-c"<<"umount -a");
        umountall.waitForFinished();
   #endif
    }
    else
    {
        qDebug() << "!! Bad password... => No update !";
    }

    umount(m_device, m_uuid);


    Q_EMIT finished();
    m_active = false;
    qDebug() << "Exit copy";
}
