#ifndef VIEWER_H
#define VIEWER_H

#include <QTimer>
#include <QProcess>
#include <QStackedWidget>
#include <QTime>
#include <QScopedPointer>

#include "runnable.h"
#include "settings.h"

#include "system/panel.h"
#include "system/shortcut.h"
#include "system/usbauto.h"


class PlaylistEventFilter : public QObject
{
public:
    PlaylistEventFilter();
    void setPaused(bool paused) { m_paused = paused; }

protected:
    bool eventFilter(QObject *obj, QEvent *event);

private:
    Settings *settings;
    bool m_paused;
};


class BrowserEventFilter : public QObject
{
protected:
    bool eventFilter(QObject *obj, QEvent *event);
};

namespace panel {
    class Manager;
}
class SystemShortcutManager;
class Webpage;

class Viewer : public QStackedWidget
{
    Q_OBJECT

public:
    Viewer();
    virtual ~Viewer();
    void mousePressEvent(QMouseEvent *);
    void emitMouseEvent();

public Q_SLOTS:
    void resetTimer();
    bool next(QWidget *widget);
    void pause();
    void showText(const QString & text, int duration);
    void showUrl(QString url, int duration);
    void takeScreen();

protected:
    void closeEvent(QCloseEvent *event);

private:
    Settings *settings;
    QTimer m_timer;
    QTime m_time;

    QScopedPointer<panel::Manager> m_panel_manager;
    QScopedPointer<SystemShortcutManager> m_system_shortcut_manager;
    QScopedPointer<UsbAuto> m_usbauto;
    QScopedPointer<QProcess> m_hideMouse;

    int remainingTime;

    QWidget *m_prevWidget = NULL;

    PlaylistEventFilter *m_playlistEventFilter = NULL;
    BrowserEventFilter *m_browserEventFilter = NULL;
    QTimer m_unlockTimer;

    void loadWindowSettings();
    void saveWindowSettings();
    void setShortcuts();
    void setTimer();
    void displayWebpage(Webpage* page);
    void killExternalPlayer();

private Q_SLOTS:
    void bye(void);
    void onWidgetTimeout(void);
    void onTimerTimeout(void);
    void stop(void);
    void unlockPlaylistEventFilter(void);
    void requestRestart(void);
    void on_currentChanged(int index);

Q_SIGNALS:
    void end(void);
    void mouseEvent(void);
    void terminate(void);
    void startInlineRemote(void);
    void restart(void);
    void readyToShoot(void);
    void rightclick(void);
    void middleclick(void);
    void leftclick(void);

};

#endif
