#ifndef BROWSER_H
#define BROWSER_H

#include <QTabWidget>
#include <QTabBar>
#include <QStatusBar>
#include <QPushButton>
#include <QVBoxLayout>
#include <QLabel>
#include <QTimer>

#include "runnable.h"
#include "processmanager.h"
#include "page.h"
#include "settings.h"
#include "tabwidget.h"

class Browser : public QWidget, public Runnable
{
    Q_OBJECT

public:
    Browser(QWidget *parent = 0);
    ~Browser(void);

    void addTime(int seconds);
    void setHome(QString url);
    void hideToolbar(void);

    QUrl getHome(void);

private:
    Settings *settings;

    TabWidget m_tabWidget;
    QStatusBar m_statusBar;
    QVBoxLayout m_layout;
    QPushButton m_cornerBtn;
    QList<Page *> m_pages;
    QUrl m_home;
    QUrl m_newTabUrl;

    int downloads;
    QLabel m_downloadLabel;

    int remaining;
    QLabel m_timerLabel;

    SubprocessManager m_processManager;

    bool startImpl(void);
    bool stopImpl(void);
    bool onTimeoutImpl(void);
    void pauseImpl(void);
    void resumeImpl(void);

    void setShortcuts();
    void setupPage(Page *p, QUrl url = QUrl());
    void refreshTabBar();

    void mousePressEvent(QMouseEvent*);

private Q_SLOTS:
    void onTitleChanged(QString &, Page *);
    void onTabCloseRequested(int);
    void onDownloadStarted();
    void onDownloadDone(QString &filepath, QString &mimetype);
    void closeCurrentTab();
    void nextTab();
    void prevTab();
    void selectAddressBar();
    void selectAddressBarAndBeginSearch();
    void onUrlDenied(QString &);
    void onPrintRequested();
    Page *newPage(QUrl url = QUrl());
    Page *newPage(RestrictedWebView *webview);
    void onMouseMove();
    void onEnd();
    void onWebViewCreated(RestrictedWebView *);

Q_SIGNALS:
    void end();
    void mouseMoved();
    void timeAdded(int);
    void rightclick();
};

#endif
