#include <QWebFrame>
#include <QWebElementCollection>
#include <QVBoxLayout>
#include <QAction>
#include <QDebug>

#include "page.h"

Page::Page(QUrl url, QWidget *parent)
    : QWidget(parent)
{
    settings = Settings::load();

    m_view = new RestrictedWebView(url);
    connect(m_view, SIGNAL(urlDenied(QString &)),
            this, SLOT(onUrlDenied(QString &)));
    setShortcut();
    setup();
}

Page::Page(RestrictedWebView *webView, QWidget *parent)
    : QWidget(parent)
{
    settings = Settings::load();
    m_view = webView;
    setup();
}

void Page::setup()
{
    m_zoomFactor = settings->get("main/zoomFactor").toDouble();
    m_navbar = new NavBar(m_view, this);
    QObject::connect(m_navbar, SIGNAL(titleChanged(QString &)),
            this, SLOT(adjustTitle(QString &)));
    QObject::connect(m_navbar, SIGNAL(end()), this, SLOT(onEnd()));

    QVBoxLayout *layout = new QVBoxLayout;
    layout->setContentsMargins(0, 0, 0, 0);
    if (!settings->get("main/hide_toolbar").toBool())
        layout->addWidget(m_navbar);
    m_view->setZoomFactor(m_zoomFactor);
    layout->addWidget(m_view);
    setLayout(layout);
    show();
}

void Page::setShortcut(){
    QAction *action;
    action = new QAction(this);
    action->setShortcuts(QList<QKeySequence>() << Qt::CTRL + Qt::Key_Plus);
    connect(action, SIGNAL(triggered()), this, SLOT(zoomPlus()));
    m_view->addAction(action);

    action = new QAction(this);
    action->setShortcuts(QList<QKeySequence>() << Qt::CTRL + Qt::Key_Minus);
    connect(action, SIGNAL(triggered()), this, SLOT(zoomMinus()));
    m_view->addAction(action);
}

void Page::onUrlDenied(QString &url)
{
    Q_EMIT urlDenied(url);
}

void Page::onEnd()
{
    Q_EMIT end();
}

void Page::zoomPlus()
{
    m_zoomFactor = m_zoomFactor + 0.1;
    m_view->setZoomFactor(m_zoomFactor);
}

void Page::zoomMinus()
{
    m_zoomFactor = m_zoomFactor - 0.1;
    m_view->setZoomFactor(m_zoomFactor);
}

void Page::adjustTitle(QString &title)
{
    Q_EMIT titleChanged(title, this);
}

void Page::removeEmbedElements()
{
    QWebElementCollection elements = m_view->page()->mainFrame()->findAllElements("embed");
    Q_FOREACH (QWebElement e, elements)
    {
        e.removeFromDocument();
    }
}
