#ifndef RESTRICTEDWEBPAGE_H
#define RESTRICTEDWEBPAGE_H

#include <QFile>
#include <QUrl>
#include <QWebPage>
#include <QRegExp>
#include <QNetworkRequest>
#include <QSslError>

#include "settings.h"

class RestrictedDownload : public QObject
{
    Q_OBJECT

public:
    RestrictedDownload(QNetworkReply *reply, QObject *parent = 0);

private:
    QString m_filepath;
    QString m_mimetype;
    QNetworkReply *m_reply;
    QFile *m_fd;

private Q_SLOTS:
    void onFinished();
    void onReadyRead();

Q_SIGNALS:
    void finished(QString &, QString &);
};

class RestrictedWebPage : public QWebPage
{
    Q_OBJECT

public:
    RestrictedWebPage(QWidget *parent = 0);
    ~RestrictedWebPage();

protected:
    QString userAgentForUrl(const QUrl &url) const;

private:
    bool acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &request, NavigationType type);

    Settings *settings;

    QList<QRegExp> regexList;
    void setupRegexList();
    bool urlIsAllowed(QUrl &);

private Q_SLOTS:
    void onDownloadRequested(const QNetworkRequest &request);
    void onUnsupportedContent(QNetworkReply *reply);
    void onDownloadDone(QString &, QString &);
    void sslErrorHandler(QNetworkReply *,
                         const QList<QSslError> &error_list);

Q_SIGNALS:
    void urlDenied(QString &);
    void downloadStarted();
    void downloadDone(QString &, QString &);
};

#endif
