#include <Qt>
#include <QDir>
#include <QFile>
#include <QUrl>
#include <QRegExp>
#include <QTextStream>
#include <QDebug>

#include "document.h"


Document::Document(PlaylistItem *item, QWidget *parent)
    : QWebView(parent)
    , Runnable(item->time())
{
    m_uri = item->uri();
    this->setContextMenuPolicy(Qt::CustomContextMenu);
}


bool Document::startImpl(void)
{
    QString basePath =
            QString("file://") + QDir::homePath() +
            QString("/.linutop-kiosk/data/");

    QFile f(m_uri.mid(7));
    if(!f.open(QIODevice::ReadOnly))
    {
        Q_EMIT end();
        return false;
    }

    QTextStream in(&f);
    QString html = in.readAll();
    f.close();

    setHtml(parseTemplate(html, basePath), QUrl(basePath));
    return true;
}


bool Document::stopImpl(void) { return true; }


bool Document::onTimeoutImpl(void)
{ Q_EMIT end(); return true; }


void Document::pauseImpl(void) {}


void Document::resumeImpl(void) {}


QString Document::parseTemplate(QString &html, QString &basePath)
{
    QRegExp limageRx("<limage([^>]*)>([^<]+)</limage>");
    int pos;

    while ((pos = limageRx.indexIn(html)) != -1) {
        QString encoded = basePath + limageRx.cap(2).toUtf8().toBase64().replace("/", "-");
        QString data = limageRx.cap(1);
        QString n = QString("<img src=\"%1\" \"%2\"/>").arg(encoded).arg(data);
        html.replace(pos, limageRx.matchedLength(), n);
    }

    return html;
}


void Document::mousePressEvent(QMouseEvent* event)
{
    switch(event->button()){
        case Qt::RightButton: {
            Q_EMIT rightclick();
            break;
        }
        default: break;
    }
}
