#include <QPixmap>
#include <QRect>
#include <QPalette>
#include <QDesktopWidget>
#include <settings.h>
#include <QMouseEvent>
#include <QtWidgets>

#include "image.h"

Image::Image(PlaylistItem *item, QWidget *parent)
    : QLabel(parent)
    , Runnable(item->time())
{
    m_uri = item->uri();
    setAlignment(Qt::AlignCenter);
    setBlackBackground();
    QRect g = qobject_cast<QWidget *>(QLabel::parent())->geometry();
    p_w = g.width();
    p_h = g.height();
    m_increment = true;
    buildPixmap();
}


void Image::setBlackBackground(void)
{
    QPalette p;
    p.setColor(QPalette::Background, Qt::black);
    QLabel::setAutoFillBackground(true);
    QLabel::setPalette(p);
}


void Image::buildPixmap(void)
{
    float s_w, s_h;
    QString path = m_uri.mid(7);
    QPixmap source(path);


    if (source.isNull())
    {
        setPixmap(source);
        return;
    }

        s_w = source.width();
        s_h = source.height();



    qDebug()<<p_w<<p_h<<s_w<<s_h;
    Settings  *settings = Settings::load();
    if(settings->get("slideshow/resize").toBool()
            && !settings->get("slideshow/auto_scroll").toBool())
    {
        pixmap = source.scaled(p_w,p_h,Qt::KeepAspectRatio, Qt::SmoothTransformation);
    }
    else if(settings->get("slideshow/auto_scroll").toBool()) // TODO : check % of scrolling
    {
        if ((s_w <= p_w) && (s_h <= p_h))
        {
            /*if(settings->get("slideshow/resize").toBool())
                pixmap = source.scaled(p_w,p_h,Qt::KeepAspectRatio, Qt::SmoothTransformation);
            else*/
                pixmap = source;
        }
        //else if((p_w/p_h < 1) && (s_w <= p_h) && (s_h <= p_w))
        /*{
            if(settings->get("slideshow/resize").toBool())
                pixmap = source.scaled(p_w,p_h,Qt::KeepAspectRatio, Qt::SmoothTransformation);
            else
                pixmap = source;
        }*/
        else
        {
            connect(&m_startTimer, SIGNAL(timeout()), this, SLOT(startScroll()));
            m_startTimer.start(500);
            scrollArea.setWidget(this);
            scrollArea.setWidgetResizable(true);
            scrollArea.hide();
            if(s_w/s_h < p_w/p_h)
            {
                qDebug() << s_w/s_h << p_w/p_h;
                pixmap = source.scaledToWidth(p_w, Qt::SmoothTransformation);
                m_screenSize = QApplication::desktop()->screenGeometry().height();
                connect(&m_scrollTimer, SIGNAL(timeout()), this, SLOT(autoScrollDown()));
            }
            else
            {
                qDebug() << s_w/s_h << p_w/p_h;
                pixmap = source.scaledToHeight(p_h, Qt::SmoothTransformation);
                m_screenSize = QApplication::desktop()->screenGeometry().width();
                connect(&m_scrollTimer, SIGNAL(timeout()), this, SLOT(autoScrollRight()));
            }
        }
    }
    else if(settings->get("slideshow/stretch").toBool())
    {
        pixmap = source.scaled(p_w,p_h, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    }
    else if ((s_w <= p_w) && (s_h <= p_h))
        pixmap = source;

    else
        pixmap = source.scaled(p_w,p_h,Qt::KeepAspectRatio, Qt::SmoothTransformation);
    setPixmap(pixmap);
}

void Image::startScroll()
{
    m_startTimer.stop();
    m_scrollTimer.start(50);
}

void Image::autoScrollDown()
{
    int newPos;
    if(m_increment)
        newPos = scrollArea.verticalScrollBar()->value()+1;
    else
        newPos = scrollArea.verticalScrollBar()->value()-1;

    scrollArea.verticalScrollBar()->setValue(newPos);

    if(newPos >= pixmap.size().height() - m_screenSize)
        m_increment = false;
    else if(newPos <= 0)
        m_increment = true;
}

void Image::autoScrollRight()
{
    int newPos;
    if(m_increment)
        newPos = scrollArea.horizontalScrollBar()->value()+1;
    else
        newPos = scrollArea.horizontalScrollBar()->value()-1;

    scrollArea.horizontalScrollBar()->setValue(newPos);

    if(newPos >= pixmap.size().width() - m_screenSize)
        m_increment = false;
    else if(newPos <= 0)
        m_increment = true;
}

bool Image::startImpl(void)
{
    if (pixmap.isNull())
    {
        m_startTimer.stop();
        Q_EMIT end();
        return false;
    }
    return true;
}

bool Image::stopImpl(void)
{
    m_scrollTimer.stop();
    return true;
}

bool Image::onTimeoutImpl(void)
{
    qDebug() << "onTimeoutImpl";
    m_scrollTimer.stop();
    Q_EMIT end();
    return true;
}

void Image::pauseImpl(void) {}


void Image::resumeImpl(void) {}

void Image::mousePressEvent(QMouseEvent* event)
{
    switch(event->button()){
        case Qt::RightButton:
            Q_EMIT rightclick();
            break;
        default: break;
    }
}
