#include <QMouseEvent>
#include <settings.h>
#include "pdf.h"


PDF::PDF(PlaylistItem *item, QWidget *parent)
    : QLabel(parent)
    , Runnable(item->time())
    , m_document(NULL)
{
    m_uri = item->uri().mid(7);
    m_document = Poppler::Document::load(m_uri);
    if (!m_document) Q_EMIT end();
    m_pageNb = -1;
}


void PDF::setBlackBackground(void)
{
    QPalette p;
    p.setColor(QPalette::Background, Qt::black);
    QLabel::setAutoFillBackground(true);
    QLabel::setPalette(p);
}


bool PDF::nextPage(void)
{
    m_pageNb++;
    m_document->setRenderHint(Poppler::Document::TextAntialiasing);
    Poppler::Page *page = m_document->page(m_pageNb);
    if (!page)
    {
        Q_EMIT end();
        return false;
    }

    QImage image = page->renderToImage(100,100);
    m_pixmap = QPixmap::fromImage(image);
    QRect g = qobject_cast<QWidget *>(QLabel::parent())->geometry();
    QPixmap pixmap;
    p_w = g.width();
    p_h = g.height();

    s_w = m_pixmap.width();
    s_h = m_pixmap.height();
    Settings  *settings = Settings::load();

    if (settings->get("slideshow/stretch").toBool())
    {
        pixmap = m_pixmap.scaled(p_w, p_h, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    }
    else if (settings->get("slideshow/resize").toBool() ||
             settings->get("slideshow/auto_scroll").toBool())
    {
        if ((s_w / p_w) > (s_h / p_h))
            pixmap = m_pixmap.scaledToWidth(p_w, Qt::SmoothTransformation);
        else
            pixmap = m_pixmap.scaledToHeight(p_h, Qt::SmoothTransformation);
    }
    else if ((s_w <= p_w) && (s_h <= p_h))
        pixmap = m_pixmap;

    setPixmap(pixmap);
    setAlignment(Qt::AlignCenter);
    setBlackBackground();

    delete page;
    return true;
}


bool PDF::startImpl(void)
{ return nextPage(); }


bool PDF::stopImpl(void)
{ return true; }


bool PDF::onTimeoutImpl(void)
{
    restart(false);
    return false;
}


void PDF::pauseImpl(void) {}


void PDF::resumeImpl(void) {}


void PDF::mousePressEvent(QMouseEvent* event)
{
    switch(event->button()){
        case Qt::RightButton:
            Q_EMIT rightclick();
            break;
        default: break;
    }
}
