#include <set>
#include <QApplication>
#include <QDesktopWidget>
#include <QVBoxLayout>
#include <QDebug>
#include <QMouseEvent>
#include <QtWidgets>
#include <settings.h>
#include <QProcess>
#include <QPixmap>
#include <QRect>

#include "./video/externalimpl.h"
#include "./video/vlcimpl.h"


Video::Video(PlaylistItem *item, QWidget *parent)
    : QWidget(parent)
    , Runnable(item->time())
{
    Settings* settings = Settings::load();

    if (settings->get("slideshow/external_player").toBool())
    {
        QString command =
                settings->get("slideshow/external_player_cmd").toString();

        QStringList cmd;
        if (!command.isEmpty())
        {
            qDebug() << "Using external command:" << command;
            cmd = command.split(' ');
        }
        else
        {
           #ifdef __arm__
             cmd << "omxplayer";
             QRect rec = QApplication::desktop()->screenGeometry();
             QString arg = "";

             if (settings->get("slideshow/mute").toBool())
                 cmd << "--vol" << "-10000";

             if(settings->get("main/fullscreen").toBool()){
                 if (settings->get("slideshow/stretch").toBool()){
                     cmd << "--win";
                     arg = "0 0 "+ QString::number(rec.width()) + " " + QString::number(rec.height());
                 }
                 else{
                     arg = "-b";
                 }
             }
             else if (settings->get("main/fixedscreen").toBool()){
                     cmd << "--win";
                     arg = "0 0 "+ QString::number(settings->get("main/largeur_edit").toInt()) + " " + QString::number(settings->get("main/hauteur_edit").toInt());
             }
             else {
                     QString width = QString::number(parent->width() + parent->geometry().x());
                     QString height = QString::number(parent->height() + parent->geometry().y());
                     cmd << "--win";
                     arg += QString::number(parent->geometry().x()) + " " + QString::number(parent->geometry().y()) + " " + width + " " + height;
             }
             cmd << arg << "%f";
            #else
            cmd << "mplayer" << "-fs" << "%f";
            #endif
        }

        m_impl = new ExternalVideoImpl(cmd, item, this);
    }
    else
        m_impl = new VLCVideoImpl(item, this);
}


Video::~Video(void)
{ delete m_impl; }


bool Video::startImpl(void)
{ return m_impl->start(); }


bool Video::stopImpl(void)
{ return m_impl->stop(); }


bool Video::onTimeoutImpl(void)
{ return m_impl->onTimeout(); }


void Video::pauseImpl(void)
{ m_impl->pause(); }


void Video::resumeImpl(void)
{ m_impl->resume(); }


void Video::mousePressEvent(QMouseEvent* event)
{
    switch(event->button()){
        case Qt::RightButton:
            Q_EMIT rightclick();
            break;
        default: break;
    }
}
