#include <set>
#include "./externalimpl.h"


#include <QProcess>
#include <QDebug>


ExternalVideoImpl::ExternalVideoImpl(
        QStringList const& command, PlaylistItem* item, Video* p)
    : VideoImpl(item, p)
    , _command(command)
    , _process(NULL)
    , _display(NULL)
{
    VideoImpl::parent()->setAutoFillBackground(true);
    QPalette plt;
    plt.setColor(QPalette::Window, Qt::black);
    VideoImpl::parent()->setPalette(plt);
}


ExternalVideoImpl::~ExternalVideoImpl(void)
{
    stop();
}


bool ExternalVideoImpl::start(void)
{
    _process = new QProcess();
    QObject::connect(
                _process, SIGNAL(finished(int)),
                this, SLOT(onMediaEndReached(int)));

    QStringList args;
    #ifdef __arm__
    QString file(uri());
    file.replace("file://", "");
    QStringList::const_iterator
            it = ++_command.begin(), itEnd = _command.end();
    for (; it != itEnd; ++it)
        args << QString(*it).replace("%f", file);
    #else
    QString file(uri());
    QStringList::const_iterator
            it = ++_command.begin(), itEnd = _command.end();
    for (; it != itEnd; ++it)
        args << QString(*it).replace("%f", file);
    args.append("video-sink=vaapisink fullscreen=true");
    #endif

    qDebug() << "Launching command:" << _command.at(0) << args;
    _process->start(_command.at(0), args);
    _process->waitForStarted();
    //QObject::connect(_process, SIGNAL(readyReadStandardOutput()), this, SLOT(readyReadStandardOutput()));
    #ifndef __arm__
    QObject::connect(_process, SIGNAL(readyReadStandardError()), this, SLOT(readyReadStandardError()));
    #endif
    return true;
}

void ExternalVideoImpl::readyReadStandardOutput(void){
    qDebug()<< _process->readAllStandardOutput();
}

bool ExternalVideoImpl::readyReadStandardError(void){
    QByteArray error = _process->readAllStandardError();
    if (error.contains("ERROR"))
    {
        qDebug() << "An error occured";
        QObject::disconnect(
                    _process, SIGNAL(finished(int)),
                    this, SLOT(onMediaEndReached(int)));
        _process->kill();
        _process->deleteLater();
        _process = NULL;
        qDebug() << "Resolved!";
        Q_EMIT end();
    }
    return true;
}


bool ExternalVideoImpl::stop(void)
{
    if (_process)
    {
        QObject::disconnect(
                    _process, SIGNAL(finished(int)),
                    this, SLOT(onMediaEndReached(int)));
        _process->deleteLater();
        _process = NULL;
#ifdef __arm__
    QProcess cmd; cmd.execute("pkill -9 omxplayer");
#endif
    }
    return true;
}


bool ExternalVideoImpl::onTimeout(void)
{
    end();
    return true;
}


void ExternalVideoImpl::onMediaEndReached(int)
{
    if (_process)
    {
        QObject::disconnect(
                    _process, SIGNAL(finished(int)),
                    this, SLOT(onMediaEndReached(int)));
        _process->deleteLater();
        _process = NULL;
        Q_EMIT end();
    }
}


#include "externalimpl.moc"
