#include "./vlcimpl.h"
#include <QUrl>



VLCVideoImpl::VLCVideoImpl(PlaylistItem* item, Video* p)
    : VideoImpl(item, p)
{
    static char const* const vlc_args[] = {
        "-I", "dummy",
        "--no-xlib",
        "--ignore-config",
        "--no-video-title-show",
        "--no-disable-screensaver",
    #ifdef __arm__
        "--vout", "omxil_vout",
    #endif
    };

    parent()->setAutoFillBackground(true);
    QPalette plt;
    plt.setColor(QPalette::Window, Qt::black);
    parent()->setPalette(plt);

    m_instance = libvlc_new(
                sizeof(vlc_args)/sizeof(*vlc_args),
                vlc_args);

    m_player = libvlc_media_player_new(m_instance);

    // Set callback on end of media.
    {
        libvlc_event_manager_t *em =
                libvlc_media_player_event_manager(m_player);

        libvlc_event_attach(
                    em, libvlc_MediaPlayerEndReached,
                    VLCVideoImpl::onMediaEndReached,
                    this);
    }
}


VLCVideoImpl::~VLCVideoImpl(void)
{
    libvlc_media_player_stop(m_player);
    libvlc_media_player_release(m_player);
    libvlc_release(m_instance);
}

void VLCVideoImpl::onMediaEndReached() { end(); }


bool VLCVideoImpl::start()
{
    QUrl url;
    if (uri().contains("file://"))
        url = QUrl::fromLocalFile(QString(uri()).replace("file://", ""));
    else if (uri().contains("vlc:"))
        url = QUrl(QString(uri()).replace("vlc:", ""));
    else
        url = uri();

    qDebug() << "Reading uri:" << url.toEncoded();
    m_media = libvlc_media_new_location(m_instance, url.toEncoded().data());
    libvlc_media_player_set_media(m_player, m_media);
#ifndef __arm__
    libvlc_media_player_set_xwindow(m_player, parent()->winId());
#endif
    libvlc_media_player_play(m_player);

    return true;
}


bool VLCVideoImpl::stop()
{
    libvlc_media_player_stop(m_player);
    libvlc_media_release(m_media);
    return true;
}


bool VLCVideoImpl::onTimeout()
{ end(); return true; }


void VLCVideoImpl::pause()
{ libvlc_media_player_pause(m_player); }


void VLCVideoImpl::resume()
{ libvlc_media_player_pause(m_player); }


void VLCVideoImpl::onMediaEndReached(const libvlc_event_t* /*e*/, void* self)
{ reinterpret_cast<VLCVideoImpl*>(self)->onMediaEndReached(); }
