#ifndef WEBPAGE_H
#define WEBPAGE_H

#include <QTimer>
#include <QWebView>
#include <QNetworkCookieJar>
#include <QUrl>
#include <QEvent>
#include <QSslError>

#include "runnable.h"
#include "playlistitem.h"
#include "settings.h"


class SqliteCookieJar : public QNetworkCookieJar
{
    Q_OBJECT

public:
    SqliteCookieJar(QObject *parent = 0);

private:
    QList<QNetworkCookie> m_cookies;
};

class CustomQWebPage : public QWebPage
{
    Q_OBJECT

protected:
    QString userAgentForUrl(const QUrl &url) const;
};

class Webpage : public QWebView, public Runnable
{
    Q_OBJECT

public:
    Webpage(PlaylistItem *item, QWidget *parent);
    bool isLoaded(){return loaded;}

protected:
    bool eventFilter(QObject *obj, QEvent *event);

private:
    CustomQWebPage m_qwebpage;
    Settings *settings;
    QUrl m_uri;
    SqliteCookieJar *m_jar;
    bool loaded;
    QTimer m_startTimer;
    QTimer m_scrollTimer;
    int m_increment;
    QStringList m_params;
    int m_scrollY;
    int m_scrollX;

    void displayHTML(PlaylistItem *item);
    bool startImpl(void);
    bool stopImpl(void);
    bool onTimeoutImpl(void);
    void pauseImpl(void);
    void resumeImpl(void);

    void mousePressEvent(QMouseEvent*);

private Q_SLOTS:
    //void onLoadFinished(bool ok);
    void sslErrorHandler(QNetworkReply*, const QList<QSslError>&);
    void startScroll();
    void autoScroll();
    void errorCheck(QNetworkReply*);
    void scrollToY();
    void scrollToX();

Q_SIGNALS:
    void end(void);
    void rightclick(void);
};

class PostSession : public Webpage
{
    Q_OBJECT
};

#endif
