#include <cstdio>
#include <unistd.h>
#include <stdlib.h>

#include <QProcess>
#include <QDebug>
#include <QCoreApplication>
#include <QObject>
#include <QRegExp>
#include <QTimer>


class Program : public QObject
{
    Q_OBJECT

    QString const& m_user;
    QString const& m_user_id;
    QString const& m_uuid;
    QString const& m_device;
    QString const& m_cmd;

public:
    Program(QObject *parent, QString const& cmd,
            QString const& user, QString const& user_id,
            QString const& uuid, QString const& device)
        : QObject(parent)
        , m_user(user)
        , m_user_id(user_id)
        , m_uuid(uuid)
        , m_device(device)
        , m_cmd(cmd) {}

public Q_SLOTS:
    void run(void)
    {
        /**/ if (m_cmd == "mount") mount();
        else if (m_cmd == "umount") umount();
        qApp->quit();
    }

private:

    void umount(void)
    {
        QString mount_path = "/media/" + m_user + "/" + m_uuid;
        QProcess umnt;
        QStringList args;
        args << mount_path;
        umnt.start("umount", args);
        qDebug() << "Running:" << "umount" << args.join(" ");
        if (!umnt.waitForFinished())
        {
            qApp->exit(-1);
            return;
        }
        else
        {
            qDebug() << umnt.readAllStandardError();
        }
    }

    void mount(void)
    {
        // Create mount directory.
        QString mount_path = "/media/" + m_user + "/" + m_uuid;
        {
            QProcess mkdir;
            mkdir.start("mkdir", QStringList() << "-p" << mount_path);
            if (!mkdir.waitForFinished())
            {
                qApp->exit(-1);
                return;
            }
        }

        // Get the fs type and mount.
        {
            QProcess blkid;
            blkid.start("blkid", QStringList() << m_device);
            blkid.waitForFinished();
            QString output = blkid.readAllStandardOutput();
            QRegExp regex("^.* TYPE=\"([^\"]*)\".*$");

            QString fs_type = output.replace(regex, "\\1");

            /**/ if (fs_type == "vfat") mount_fat(mount_path);
            else if (fs_type == "ext2") mount_default(mount_path);
            else if (fs_type == "ext3") mount_default(mount_path);
            else if (fs_type == "ext4") mount_default(mount_path);
            else
            {
                qDebug() << "Unsupported fs type:" << fs_type;
                qDebug() << "Trying default mounting pattern...";
                mount_default(mount_path);
            }
        }
    }

    void mount_default(QString const& mount_path)
    {
        QProcess mnt;
        QStringList args;
        args << m_device << mount_path;
        mnt.start("mount", args);
        qDebug() << "Running:" << "mount" << args.join(" ");
        if (!mnt.waitForFinished())
        {
            qApp->exit(-1);
            return;
        }
        else
        {
            qDebug() << mnt.readAllStandardError();
        }
    }

    void mount_fat(QString const& mount_path)
    {
        QProcess mnt;
        mnt.start("mount", QStringList()
                  << "-o" << ("uid=" + m_user_id) << m_device << mount_path);
        if (!mnt.waitForFinished())
        {
            qApp->exit(-1);
            return;
        }
        else
        {
            qDebug() << mnt.readAll();
        }
    }

};

int main(int argc, char** argv)
{
    if (setuid(0) != 0)
    {
        qDebug() << "Unable to be root.";
        return -2;
    }

    if (argc < 6) return -1;

    QCoreApplication app(argc, argv);

    QString user = argv[1];
    QString user_id = argv[2];
    QString uuid = argv[3];
    QString device = argv[4];
    QString cmd = argv[5];

    if (cmd != "mount" && cmd != "umount") return -3;

    qDebug() << user << user_id << uuid << device;

    Program* pgrm = new Program(&app, cmd, user, user_id, uuid, device);
    QTimer::singleShot(0, pgrm, SLOT(run()));

    return app.exec();
}

#include "main.moc"
