#include "commandlineparser.h"

#include <iostream>
#include <QString>
#include <QDebug>


CommandLineParser::CommandLineParser(int argc, const char * const *argv)
    : m_base_path(defaultBasePath())
    , m_program_name(argv[0])
    , m_need_help(false)
{
    for (int argn = 1; argn < argc; ++argn)
    {
        QString current(argv[argn]);
        /**/ if (current == preferenceFilenameArgument())
        {
            if (++argn < argc) m_preference_filename = argv[argn];
            else return;
        }
        else if (current == playlistFilenameArgument())
        {
            if (++argn < argc) m_playlist_filename = argv[argn];
            else return;
        }
        else if (current == whiteListFilenameArgument())
        {
            if (++argn < argc) m_white_list_filename = argv[argn];
            else return;
        }
        else if (current == cookieFilenameArgument())
        {
            if (++argn < argc) m_cookie_filename = argv[argn];
            else return;
        }
        else if (current == basePathArgument())
        {
            if (++argn < argc) m_base_path = argv[argn];
            else return;
        }
        else if (current == hookDirnameArgument())
        {
            if (++argn < argc) m_hook_dirname = argv[argn];
            else return;
        }
        else if (current == helpArgument())
        {
            m_need_help = true;
        }
        else
        {
            qDebug() << "Unrecognized argument: "
                     << current.toStdString().c_str();
        }
    }
}

QString CommandLineParser::getPreferenceFilename(void) const
{
    return m_preference_filename.isEmpty()
            ? defaultPreferenceFilename(m_base_path)
            : m_preference_filename;
}


QString CommandLineParser::getPlaylistFilename(void) const
{
    return m_playlist_filename.isEmpty()
            ? defaultPlaylistFilename(m_base_path)
            : m_playlist_filename;
}

QString CommandLineParser::getWhiteListFilename(void) const
{
    return m_white_list_filename.isEmpty()
            ? defaultWhiteListFilename(m_base_path)
            : m_white_list_filename;
}

QString CommandLineParser::getCookieFilename(void) const
{
    return m_cookie_filename.isEmpty()
            ? defaultCookieFilename(m_base_path)
            : m_cookie_filename;
}

QString CommandLineParser::getHookDirname(void) const
{
    return m_hook_dirname.isEmpty()
            ? defaultHookDirname(m_base_path)
            : m_hook_dirname;
}

void CommandLineParser::displayHelp(void) const
{
    int table_size = 15;
    std::cout << m_program_name.toStdString() << std::endl
              << "Arguments:" << std::endl
              << "  " << helpArgument().leftJustified(table_size)
                         .toStdString()
                      << "Display this help menu and exit" << std::endl
              << "  " << basePathArgument().leftJustified(table_size)
                         .toStdString()
                      << "Use this as default base path (default="
                      << defaultBasePath().toStdString() << ")" << std::endl
              << "  " << preferenceFilenameArgument().leftJustified(table_size)
                         .toStdString()
                      << "Use this as path for preference file (default="
                      << defaultPreferenceFilename("<base_path>").toStdString()
                      << ")" << std::endl
              << "  " << playlistFilenameArgument().leftJustified(table_size)
                         .toStdString()
                      << "Use this as path for playlist file (default="
                      << defaultPlaylistFilename("<base_path>").toStdString()
                      << ")" << std::endl
              << "  " << cookieFilenameArgument().leftJustified(table_size)
                         .toStdString()
                      << "Use this as cookie file for the browser (default="
                      << defaultCookieFilename("<base_path>").toStdString()
                      << ")" << std::endl
              << "  " << hookDirnameArgument().leftJustified(table_size)
                         .toStdString()
                      << "Use this directory to find hooks (default="
                      << defaultHookDirname("<base_path>").toStdString()
                      << ")" << std::endl;
}
