#ifndef COMMANDLINEPARSER_H
#define COMMANDLINEPARSER_H

#include <QString>

class CommandLineParser
{
public:
    CommandLineParser(int argc, char const* const* argv);

    QString getPreferenceFilename(void) const;
    QString getPlaylistFilename(void) const;
    QString getWhiteListFilename(void) const;
    QString getCookieFilename(void) const;
    QString getHookDirname(void) const;
    bool needHelp(void) const { return m_need_help; }
    void displayHelp(void) const;

    static QString preferenceFilenameArgument(void)
    { return QString("--preference"); }

    static QString playlistFilenameArgument(void)
    { return QString("--playlist"); }

    static QString whiteListFilenameArgument(void)
    { return QString("--whitelist"); }

    static QString cookieFilenameArgument(void)
    { return QString("--cookie"); }

    static QString basePathArgument(void)
    { return QString("--basepath"); }

    static QString hookDirnameArgument(void)
    { return QString("--hooks"); }

    static QString helpArgument(void)
    { return QString("--help"); }

    static QString defaultBasePath(void)
    { return "/etc/xdg/webkiosk"; }

    static QString defaultPreferenceFilename(
            QString const& base = defaultBasePath())
    { return base + "/preferences"; }

    static QString defaultPlaylistFilename(
            QString const& base = defaultBasePath())
    { return base + "/playlist"; }

    static QString defaultIniPlaylistFilename(
            QString const& base = defaultBasePath())
    { return base + "/playlist-ini"; }

    static QString defaultWhiteListFilename(
            QString const& base = defaultBasePath())
    { return base + "/list"; }

    static QString defaultCookieFilename(
            QString const& base = defaultBasePath())
    { return base + "/cookies.sqlite"; }

    static QString defaultHookDirname(
            QString const& base = defaultBasePath())
    { return base + "/hooks"; }

private:

    QString m_preference_filename;
    QString m_playlist_filename;
    QString m_white_list_filename;
    QString m_cookie_filename;
    QString m_hook_dirname;
    QString m_base_path;
    QString m_program_name;
    bool m_need_help;
};

#endif // COMMANDLINEPARSER_H
