#include "proxyfactory.h"

ProxyFactory::ProxyFactory(const CommandLineParser *cmdparse)
    : QNetworkProxyFactory()
{
    settings = Settings::load(false, cmdparse);
    proxyType = settings->get("network/proxy_type").toInt();

    if (proxyType == 1)
    {
        qputenv("http_proxy", "wpad://");
        qputenv("https_proxy", "wpad://");
        qputenv("ftp_proxy", "wpad://");
    }

    pf = px_proxy_factory_new();
    proxyUser = settings->get("network/proxy_user").toString();
    proxyPasswd = settings->get("network/proxy_passwd").toString();
    proxyUri = QUrl(settings->get("network/proxy_uri").toString());
}

ProxyFactory::~ProxyFactory()
{
    px_proxy_factory_free(pf);
}

QList<QNetworkProxy> ProxyFactory::queryProxy(const QNetworkProxyQuery &query)
{
    QList<QNetworkProxy> l;

    if (proxyType == 1)
    {
        char **proxies = px_proxy_factory_get_proxies(pf, query.url().toString().toUtf8().data());
        for (char **p = proxies; *p; p++)
        {
            if (QString(*p).toLower().startsWith("direct"))
            {
                l << QNetworkProxy(QNetworkProxy::NoProxy);
                break;
            }

            QUrl u(*p);
            int port = u.port();
            if (port == -1)
                port = 8080;
            l << QNetworkProxy(QNetworkProxy::HttpProxy,
                               u.host(), port, proxyUser, proxyPasswd);
        }

        for (char **p = proxies; *p; p++)
            free (*p);
        free (proxies);

        return l;
    }

    if (proxyType == 2)
    {
        int port = proxyUri.port();
        if (port == -1)
            port = 8080;
        l << QNetworkProxy(QNetworkProxy::HttpProxy,
                           proxyUri.host(), port, proxyUser, proxyPasswd);

    }

    /* no proxy */
    l << QNetworkProxy(QNetworkProxy::NoProxy);
    return l;
}
