#ifndef RUNNABLE_H
#define RUNNABLE_H

#include <QTime>
#include <QTimer>
#include <QDebug>
#include <QObject>


class Runnable;
class RunnableTimer : QTimer
{
    Q_OBJECT

public:

    RunnableTimer(Runnable* run, QObject* parent = NULL);

private Q_SLOTS:

    void onTimeout(void);

private:

    friend class Runnable;

    Runnable* m_run;
};

class Runnable
{
public:

    Runnable(int time);
    virtual ~Runnable(void) {}

    void restart(bool force_stop = true);
    bool start(void);
    bool pause(void);
    void stop(void);
    bool isActive();

    void setTimeout(int time);

private:

    friend class RunnableTimer;

    virtual bool stopImpl(void) = 0;
    virtual bool startImpl(void) = 0;
    virtual bool onTimeoutImpl(void) = 0;
    virtual void pauseImpl(void) = 0;
    virtual void resumeImpl(void) = 0;

    RunnableTimer m_timer;
    int m_time;
    bool m_started;
};


#endif
