#ifndef SETTINGS_H
#define SETTINGS_H

#include <QObject>
#include <QStringList>
#include <QVariant>
#include <QMap>
#include <QNetworkCookie>


class CommandLineParser;
class Settings : public QObject
{
    Q_OBJECT

public:
    static Settings *load(
            bool force=false, CommandLineParser const* cmdparse = NULL);

    void save(bool with_autostart,
              QString const& settings,
              QString const& playlist,
              QString const& restrictions,
              QString const& cookies);
    void save(bool with_autostart, QString const& basePath);
    void save(bool with_autostart = true);

    QVariant get(QString const& key) const;
    void set(QString const& key, QString value);
    void set(QString const& key, bool value);
    void set(QString const& key, int value);
    void set(QString const& key, double value);
    void populateDefaults();
    QString preferencesFilename(void) const;
    QString playlistFilename(void) const;
    QString whileListFilename(void) const;
    QString cookiesFilename(void) const;
    QString hookDirname(void) const;

    CommandLineParser const* getCommandLineParser(void) const
    { return m_cmdparse; }

    bool versionChanged(void) const { return m_version_changed; }

    bool autostarted(void) const { return m_autostarted; }
    void setAutostarted(bool autostarted) { m_autostarted = autostarted; }

    QStringList restrictions;
    QStringList playlistItems;
    QList<QNetworkCookie> cookiesList;

    void dump();

private:

    static Settings *m_settings;

    Settings(const CommandLineParser *cmdparse);

    CommandLineParser const* m_cmdparse;
    QMap<QString, QVariant> m_map;
    bool m_autostarted;
    bool m_version_changed;


    void readAutostarted();
    void writeAutostarted();
    void removeAutostart();
    void enableScreenSaver(bool enable);
    bool writePlaylist(QString const& where);
    bool writeRestrictions(QString const& where);
    bool writeCookies(QString const& where);
    bool writePreferences(QString const& where);
};

#endif /* SETTINGS_H */
