#include <QProcess>

#include <QDebug>

#include <unistd.h>
#include <sys/vfs.h>

#include "settings.h"
#include "utils.h"

QString Utils::getIconTheme()
{
    QProcess p;
    #ifdef __arm__
        p.start("gksu --u pi \"xfconf-query -c xsettings -p /Net/IconThemeName\"");
    #else
        p.start("xfconf-query -c xsettings -p /Net/IconThemeName");
    #endif
    p.waitForFinished();
    QString r(p.readAllStandardOutput());
    if (r.simplified() == "") {
        return "elementary-xfce";
    }
    else{
        return r.simplified();
    }
}

bool Utils::validateAdminPassword(QString given)
{
    Settings *settings = Settings::load();
    QString cryptedPassword = settings->get("main/admin_passwd").toString();

    if (cryptedPassword.isEmpty())
        return true;

    QString crypted = QString(crypt(given.toUtf8().data(), cryptedPassword.toUtf8().data()));

    if (crypted == cryptedPassword)
        return true;

    return false;
}

/* From QtMobility */
qlonglong Utils::availableDiskSpace()
{
    struct statfs fs;
    if (statfs("/", &fs) == 0) {
        qlonglong blockSize = fs.f_bsize;
        qlonglong availBlocks = fs.f_bavail;
        return availBlocks * blockSize;
    }

    return 0;
}

qlonglong Utils::totalDiskSpace()
{
    struct statfs fs;
    if (statfs("/", &fs) == 0) {
        qlonglong blockSize = fs.f_bsize;
        qlonglong totalBlocks = fs.f_blocks;
        return totalBlocks * blockSize;
    }

    return 0;
}

char Utils::genChar()
{
    static std::string charset = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
    return charset[rand() % charset.size()];
}

std::string Utils::genString(size_t length)
{
    char rbuf[length];
    std::generate(rbuf, rbuf+length, &Utils::genChar);
    return std::string(rbuf, length);
}
