#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, sys, pwd, socket, time

import json
from urlparse import urlparse

import bottle
from bottle_logging import logger
import requests

import mimetypes

from settings import Settings

UDP_IP = '127.0.0.1'
UDP_PORT = 54545
BUFFER_SIZE = 1024
PAUSE = 0
NEXT = 1
GOTO = 2
SCREENSHOT = 3
RESTART = 4
TEXT = 5
LOOP = 6
URL = 7
DELETE_DRAG_DROP = 8

localFilePlaylists = ["_"]
currentLocalFilePlaylist = 0

socketclient = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])))
settings = Settings()

config_directory    = os.path.join(sys.argv[1], ".linutop-kiosk")
dragndrop_directory = os.path.join(config_directory, "dragndrop")
usbauto_directory   = os.path.join(config_directory, "usbauto")

def getLocalPlaylist():
    plist = []
    lst = []

    # Add drag and drop directory if exists.
    if os.path.isdir(dragndrop_directory):
        flist = sorted(os.listdir(dragndrop_directory), key=lambda s: s.lower())
        for ditem in flist:
            lst.append((ditem, True))

    # Add playlist from playlist configuration file.
    with open("/etc/xdg/webkiosk/playlist") as fd:
        for line in fd.readlines():
            lst.append((line, False))

    for item in lst:

        if item[0][0] == "#":
            continue

        name = item[0].split(' ')[0]
        deletable = item[1]

        # Special case for usb auto.
        if name == "usb-auto":
            if os.path.isdir(usbauto_directory):
                flist = sorted(os.listdir(usbauto_directory), key=lambda s: s.lower())
                flist.reverse()
                for ditem in flist:
                    lst.insert(lst.index(item)+1, (ditem, False))

        # Special case for directories.
        elif os.path.isdir(urlparse(name).path):
            flist = sorted(os.listdir(urlparse(name).path), key=lambda s: s.lower())
            flist.reverse()
            for ditem in flist:
                lst.insert(lst.index(item)+1, (ditem, False))

        # Standard case: file.
        else:
            item = {
                "name": name,
                "deletable": deletable,
            }
            plist.append(item)

    return plist

@bottle.route('/control/pause', method="POST")
def play():
    socketclient.sendto(json.dumps({"action": PAUSE}),
                             (UDP_IP, UDP_PORT))

@bottle.route('/dragndrop/upload', method='POST')
def addDragAndDrop():
    try:
        # Create files as simple user.
        uid = pwd.getpwnam(sys.argv[2])[2]
        os.seteuid(uid)

        logger.debug(bottle.request.files)
        for key in bottle.request.files:
            f = bottle.request.files.get(key)
            if not os.path.isdir(dragndrop_directory):
                os.makedirs(dragndrop_directory)
            registration_path = os.path.join(dragndrop_directory, f.filename)
            if (os.path.exists(registration_path)):
                os.remove(registration_path)
            f.save(registration_path)
            logger.info("Upload registered at: %s" % registration_path)
        to_send = json.dumps({"action": RESTART})
        socketclient.sendto(to_send, (UDP_IP, UDP_PORT))
    finally:
        # Switch back to root.
        os.seteuid(0)

@bottle.route('/dragndrop/delete/<filepath:path>', method='POST')
def deleteDragAndDrop(filepath):
    socketclient.sendto(json.dumps({
        "action": DELETE_DRAG_DROP,
        "data": {
            "path": filepath
        }}),
        (UDP_IP, UDP_PORT))

@bottle.route('/remote/goto', method="POST")
def goto():
    if len(bottle.request.forms.filePath) > 0:
        socketclient.sendto(json.dumps({
            "action" : GOTO,
            "data" : { "filePath" : bottle.request.forms.filePath }
        }), (UDP_IP, UDP_PORT))
    else:
        data = {"index": bottle.request.forms['index'],
                "loop": bottle.request.forms["loop"]}
        socketclient.sendto(json.dumps({"action": GOTO, "data": data}),
                                 (UDP_IP, UDP_PORT))

@bottle.route('/control/next', method="POST")
def next():
    socketclient.sendto(json.dumps({"action": NEXT}),
        (UDP_IP, UDP_PORT))

@bottle.route('/remote/screenShot', method="POST")
def screenShot():
    socketclient.sendto(json.dumps({"action": SCREENSHOT}),
        (UDP_IP, UDP_PORT))

@bottle.route('/remote/restart', method='POST')
def restart():
    to_send = json.dumps({"action": RESTART})
    socketclient.sendto(to_send, (UDP_IP, UDP_PORT))

@bottle.route('/remote/start', method='POST')
def start():
    serverType, url, invoice, email = settings.getServerType()
    authenticated = invoice != "" and email != ""
    logger.debug("Server type: %s (%s)" % (
        serverType,
        "authenticated" if authenticated else "public"
    ))
    if bottle.request.forms['screen'] == 'local':
        settings.setLocal()
    else:
        channel = url + '/webapp/p/' + bottle.request.forms['screen']
        settings.setChannel(serverType, channel)
    to_send = json.dumps({"action": RESTART})
    socketclient.sendto(to_send, (UDP_IP, UDP_PORT))

@bottle.route('/res/screen')
def screen():
    return bottle.static_file("/tmp/screen.png", root='/')

@bottle.route('/remote/text', method="POST")
def text():
    for form in bottle.request.forms:
        text = bottle.request.forms.get('text')

        # Url display request.
        if text.startswith("http://") or text.startswith("https://"):
            split = text.split()
            if len(split) == 0: return
            url = split[0]
            if len(split) > 1:
                duration = split[1]
            else:
                duration = -1

            logger.info("URL display requested at: %s", text)
            socketclient.sendto(json.dumps({
                "action": URL,
                "data": {
                    "url"     : url,
                    "duration": duration,
                }
            }), (UDP_IP, UDP_PORT))
        # Text display request.
        else:
            data = {"text": text}
            logger.info("Text display requested: \"%s\"" % text)
            socketclient.sendto(json.dumps({"action": TEXT, "data": data}),
                                 (UDP_IP, UDP_PORT))

@bottle.route('/static/<filepath:path>')
def server_static(filepath):
    return bottle.static_file(filepath, root='./web/static')

@bottle.route("/info")
def info():
    url = "local"
    invoice = "local"
    if settings.getRemote():
        serverType, url, invoice, email = settings.getServerType()

        if serverType and url and invoice and email:
            logger.info("Logging into: " + str(serverType))
            logger.info("  url:     "    + str(url))
            #logger.info("  invoice: "    + str(invoice))
            #logger.info("  email:   "    + str(email))

            headers = {
                "Content-type": "application/x-www-form-urlencoded; charset=UTF-8",
                "Accept": "application/json"
            }
            url += "/webapp/api/user/" + invoice
            data = {"model": json.dumps({"invoice": invoice, "email": email})}

            response = requests.post(url, data=data, headers=headers)
            if response.status_code != requests.codes.ok: invoice = "public"

    return json.dumps({"url": url, "invoice": invoice})

@bottle.route('/remote')
def remote():
    return bottle.static_file('remote.html', root='./web')

@bottle.route('/remote/channels')
def channels():
    global localFilePlaylists
    global currentLocalFilePlaylist

    if not settings.isTv():
        logger.info("Tv is not activated.")
        return json.dumps({ "isPrev" : False , "isTrue" : False , "data" : [] })

    if bottle.request.query.q:
        if bottle.request.query.q == "next":
            if currentLocalFilePlaylist < len(localFilePlaylists) - 1:
                currentLocalFilePlaylist += 1
        elif bottle.request.query.q == "prev":
            if currentLocalFilePlaylist > 0:
                currentLocalFilePlaylist -= 1

    cur = localFilePlaylists[currentLocalFilePlaylist]

    if isinstance(cur, list):
        invoice = ""
        response = []
        for screen in cur:
            (login, sc) = screen[1].split('/')[-2].split('-')
            response.append({
                "label" : screen[0],
                "login" : login,
                "screen" : sc,
                "token" : screen[1].split('/')[-1]
            })
    else:
        if cur != "_":
            (invoice, password) = cur
            serverType = "serverLinutopTV"
            baseurl = "http://www.linutop.tv"
        else:
            serverType, baseurl, invoice, password = settings.getServerType()

        logger.info("Server type: %s" % serverType)

        headers = {"Content-type":
                   "application/x-www-form-urlencoded; charset=UTF-8",
                   "Accept": "application/json"}
        url = baseurl
        url += "/webapp/api/user/" + invoice
        data = {"model": json.dumps({"invoice": invoice, "email": password})}

        logger.info("Data: %s" % data)
        logger.info("Getting channels at: %s" % url)

        response = requests.post(url, data=data, headers=headers)
        if response.status_code != requests.codes.ok:
            logger.info("Response code: %d" % response.status_code)
            if hasattr(response, "text"):
                logger.info("Response: %s" % str(response.text))
            logger.info("Trying to get public channels...")
            response = requests.get(baseurl + "/webapp/db/channels", headers=headers)
            if response.status_code != requests.codes.ok:
                logger.info("Response code: %d" % response.status_code)
                logger.info("Response: %s" % str(response.text))
                logger.info("Unable to get valid response...")
                return "{}"

        logger.info("Authentication success.")

        if hasattr(response, "text"):
            logger.info("Response: %s" % response.text)

        if callable(getattr(response, "json")):
            response = response.json()
        else:
            response = response.json

    r = {
        "isPrev" : currentLocalFilePlaylist > 0,
        "isNext" : currentLocalFilePlaylist < len(localFilePlaylists) - 1,
        "invoice" : invoice,
        "data" : response
    }

    return json.dumps(r)

@bottle.route('/remote/localfiles')
def localfiles():
    path = bottle.request.query.path
    logger.info("Get local files from: %s" % str(path))
    files = []
    if (settings.isLocalFs()):
        try:
            if os.path.isdir(path):
                for f in os.listdir(path):
                    if f[0] == '.':
                        continue
                    ignore = (f != "media" and f != "home")
                    logger.info("Ignoring %s: %s" % (f, ignore))
                    if path == "/" and ignore:
                        continue
                    if os.path.isdir(os.path.join(path, f)):
                        files.append({ "name" : f , "directory" : True })
                    else:
                        ft = None
                        (mt, _) = mimetypes.guess_type(os.path.join(path, f))
                        if mt != None:
                            if mt.find('image/') == 0:
                                ft = 'image'
                            elif mt.find('video/') == 0:
                                ft = 'video'
                            elif mt.find('audio/') == 0:
                                ft = 'audio'
                            elif mt == "application/pdf" or mt == "application/x-pdf":
                                ft = 'document'
                        if ft != None:
                            files.append({ "name" : f , "directory" : False , "type" : ft })
        except:
            return json.dumps([])
        files = sorted(sorted(files, key=lambda f: f['name']), key=lambda f: not f['directory'])
    return json.dumps(files)

@bottle.route('/remote/playlist')
def playlist():
    playlist = ""
    is_remote = False
    if settings.getRemote():
        is_remote = True
        path = os.path.join(config_directory, "playlist")
        with open(path) as fd:
            playlist = fd.read()
    else:
        playlist = getLocalPlaylist()
    reply = json.dumps({"playlist": playlist, "remote": is_remote})
    return reply

@bottle.route('/text')
def ddonly():
    return bottle.static_file("text.html", root="./web")

@bottle.route('/')
def home():
    if bottle.request.query.k == settings.getKey():
        if settings.getDirect():
            return ddonly()
        return bottle.static_file('index.html', root='./web')
    return bottle.Response('', 403)

@bottle.route('/remote/islocalplaylist')
def isLocalPlaylist():
    return json.dumps({ "data" : settings.isLocalPlaylist() })

if __name__ == "__main__":
    localFilePlaylists = localFilePlaylists + settings.getLocalPlaylistFile()
    currentLocalFilePlaylist = 0
    logger.setup(bottle)
    logger.info("Set application in debug mode.")
    logger.info("Home dir: %s" % sys.argv[1])
    logger.info("User: %s" % sys.argv[2])
    logger.info("localFilePlaylists: %s" % str(localFilePlaylists))
    bottle.debug(True)
    try:
        bottle.run(host='0.0.0.0', port=7575)
    except Exception as e:
        logger.error(str(e))
