#include <QFileDialog>
#include <QDebug>
#include <QIcon>
#include <QProcess>

#include <QMessageBox>
#include "utils.h"
#include <QtWidgets/qwidget.h>
#include "addplaylistitem.h"
#include <settings.h>
#include <QRegExp>

AddPlaylistItemDialog::AddPlaylistItemDialog(int time, QWidget *parent)
    : QDialog(parent)
{
    ui.setupUi(this);

    ui.timeSpin->setValue(time);

    connect(ui.AddfileBtn, SIGNAL(clicked()), this, SLOT(onFileSelected()));

    ui.AddfileBtn->setIcon(QIcon::fromTheme("applications-multimedia"));
    ui.AddfolderBtn->setIcon(QIcon::fromTheme("fileopen"));
    ui.AddnetworkBtn->setIcon(QIcon::fromTheme("folder-new"));
    ui.AddUrlBtn->setIcon(QIcon::fromTheme("window-new"));
    ui.AddPanelIcon->setPixmap(QPixmap("/usr/share/icons/elementary-xfce/actions/32/add.png"));

    ui.buttonBox->button(QDialogButtonBox::Ok)->setIcon(QIcon::fromTheme("gtk-ok"));
    ui.buttonBox->button(QDialogButtonBox::Cancel)->setIcon(QIcon::fromTheme("gtk-cancel"));
    ui.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
}

void AddPlaylistItemDialog::onFileSelected()
{
    QString path = QFileDialog::getOpenFileName(this, "Open a File");
    if (! path.isEmpty())
        ui.uriEdit->setText(QString("file://%1").arg(path));
}

void AddPlaylistItemDialog::on_AddfolderBtn_clicked()
{
    QString path = QFileDialog::getExistingDirectory(this, "Open a Folder");
    qDebug() << path;
    if (!path.isEmpty())
    {
        if(path.contains("smb:"))
        {
            int idx = path.indexOf("smb:");
            QString pth = path.mid(idx);
            pth.insert(4,"/");
            ui.uriEdit->setText(pth);
            qDebug() << pth;
        }
        else
            ui.uriEdit->setText(QString("file://%1").arg(path));
    }
}

void AddPlaylistItemDialog::on_AddnetworkBtn_clicked()
{
    QMessageBox::information(
                        NULL, QObject::tr("Configuring network access"),
                        QObject::tr("Open the file browser located in the accessory menu.\n"
                                    "Click on the network icon and navigate to the network directory.\n"
                                    "Validate and maintain network authentication.\n"
                                    "Copy the path (Network: /// ...) on the URI location and confirm.\n"));
}

void AddPlaylistItemDialog::on_AddUrlBtn_clicked()
{
    QProcess process;
    #ifndef __arm__
    process.start("firefox --new-window https://www.google.com/");
    #else
    process.start("chromium-browser -kiosk https://www.google.com/");
    #endif
    process.waitForFinished(-1);

}

void AddPlaylistItemDialog::on_uriEdit_textChanged(const QString &text){

    QString verif = ui.uriEdit->text().toLower();
    if(verif.contains(QRegExp("^http://|^files:///|^smb://|^rtsp://|^file:///|https://"))){
    ui.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
    }else{
    ui.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    }
}
