#include <unistd.h>

#include <QFileDialog>
#include <QFontDialog>
#include <QFile>
#include <QTextStream>
#include <QIcon>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QNetworkProxyFactory>
#include <QNetworkAccessManager>
#include <QtScript>
#include <QtSql>
#include <QInputDialog>
#include <QMessageBox>
#include <QApplication>
#include <QAction>

#include <QDebug>

#include "cookiesaver.h"
#include "password.h"
#include "addplaylistitem.h"
#include "addurl.h"
#include "utils.h"
#include "settingsui.h"
#include "proxyfactory.h"
#include "commandlineparser.h"
#include "processoutputreader.h"

// empeche une nbr infini de reset
int nbreset = 0;

SettingsDialog::SettingsDialog(
        QWidget *parent, const CommandLineParser* cmdparse)
    : QDialog(parent)
    , m_isInit(true)
{
    QApplication::setStyle("GTK+");
    ui.setupUi(this);

    settings = Settings::load(true, cmdparse);
    m_testProcess = 0;

    QIcon::setThemeName(Utils::getIconTheme());
    finalizeUi();
    applySettings();
    m_isInit = false;
}

void SettingsDialog::finalizeUi()
{
    // FIXME: don't hardcode /share/ path
    ui.headerLabel->setPixmap(QPixmap("/usr/share/pixmaps/linutop-webkiosk.png"));

    ui.startupChk->setChecked(settings->autostarted());
    ui.passwordBtn->setEnabled(false);
    ui.restrictionDeleteBtn->setEnabled(false);

    QItemSelectionModel *model = ui.restrictionsListView->selectionModel();
    connect(model, SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)),
            this, SLOT(on_restrictionsSelection_changed(const QItemSelection &, const QItemSelection &)));

    m_localPlaylistView = new TreeView(this);
    ui.playlistLayout->insertWidget(0, m_localPlaylistView, 1);
    connect(m_localPlaylistView, SIGNAL(dataChanged()),
            this, SLOT(genPlaylistList()));

    ui.localPlaylistDeleteBtn->setEnabled(false);
    ui.localPlaylistUpBtn->setEnabled(false);
    ui.localPlaylistDownBtn->setEnabled(false);
    ui.localPlaylistScreenOffBtn->setEnabled(true);

    // icons for buttons
    ui.restrictionAddBtn->setIcon(QIcon::fromTheme("list-add"));
    ui.restrictionDeleteBtn->setIcon(QIcon::fromTheme("list-remove"));
    ui.restrictionImportBtn->setIcon(QIcon::fromTheme("document-import"));
    ui.restrictionExportBtn->setIcon(QIcon::fromTheme("document-export"));

    ui.localPlaylistAddBtn->setIcon(QIcon::fromTheme("list-add"));
    ui.localPlaylistDeleteBtn->setIcon(QIcon::fromTheme("list-remove"));
    ui.localPlaylistUpBtn->setIcon(QIcon::fromTheme("go-up"));
    ui.localPlaylistDownBtn->setIcon(QIcon::fromTheme("go-down"));
    ui.localPlaylistScreenOffBtn->setIcon(QIcon::fromTheme("preferences-desktop-screensaver"));
    ui.localPlaylistImportBtn->setIcon(QIcon::fromTheme("document-import"));
    ui.localPlaylistExportBtn->setIcon(QIcon::fromTheme("document-export"));
    ui.localPlaylistCookiesBtn->setIcon(QIcon::fromTheme("application-x-bittorrent"));


    ui.remotepassword->setText(settings->get("remote/key").toString());

    ui.okBtn->setIcon(QIcon::fromTheme("gtk-ok"));
    ui.cancelBtn->setIcon(QIcon::fromTheme("gtk-cancel"));
    ui.helpBtn->setIcon(QIcon::fromTheme("gtk-help"));
    ui.testBtn->setIcon(QIcon::fromTheme("stock_media-play"));

    QStringList dataArray;
    dataArray << "false"<<"1;d"<<"3;d"<<"7;d"<<"21;d"<<"1;m"<<"3;m";
    foreach (QString data, dataArray) {
        ui.cleanAfter->setItemData(dataArray.indexOf(data),data);
    }
#ifdef DEMO
    ui.homeEdit->setEnabled(false);
    ui.label_2->setText(tr("<html><head/><body><p><span style='font-size:12pt; font-weight:600;'>Linutop Kiosk - Setup - Demo</span><span style='font-size:12pt;'><br/></span><span style='font-style:italic;'>For Secure Internet Access and Digital Signage</span></p></body></html>"));
#endif

    // temporary
    ui.localPlaylistImportBtn->setVisible(false);
    ui.localPlaylistExportBtn->setVisible(false);
    ui.cleanLabel->setVisible(false);
    ui.cleanAfter->setVisible(false);
}

void SettingsDialog::setText(QLineEdit *obj, const char *path)
{
    obj->setText(settings->get(path).toString());
}

void SettingsDialog::setChecked(QCheckBox *obj, const char *path, bool value)
{
    obj->setChecked(settings->get(path).toBool() == value);
}

void SettingsDialog::setSpin(QSpinBox *obj, const char *path)
{
    obj->setValue(settings->get(path).toInt());
}

void SettingsDialog::setZoom(QDoubleSpinBox *obj, const char *path)
{
    obj->setValue(settings->get(path).toDouble());
}


void SettingsDialog::applySettings()
{
    QTabBar *bar = ui.tabWidget->tabBar();

    if (settings->get("main/mode") == "kiosk")
    {
        if (settings->get("main/after_web_session") == "session")
        {
            ui.useKioskRadio->setChecked(true);
            bar->setTabEnabled(1, true);
            bar->setTabEnabled(2, false);
        }
        else
        {
            ui.useBothChk->setChecked(true);
            bar->setTabEnabled(1, true);
            bar->setTabEnabled(2, true);
        }
    }
    else
    {
        ui.useDisplayChk->setChecked(true);
        bar->setTabEnabled(1, false);
        bar->setTabEnabled(2, true);
    }

    QString updateMode =
            settings->get("slideshow/playlist_update_mode").toString();
    radioChoiceInit();
    if (updateMode == "localPlaylist")
    {
        ui.updatePlaylistRadio->setChecked(true);
        setLocalPlaylist();
    }
    else if (updateMode == "serverLinutopTV")
    {
        ui.updateOnlineRadio->setChecked(true);
        setLinutopTV(false);
    }
    else if (updateMode == "serverLinutopTVPrivate")
    {
        ui.updateOnlineRadio->setChecked(true);
        ui.privateServer->setChecked(true);
        setLinutopTV(true);
    }
    else if (updateMode == "serverPlaylistPrivate")
    {
        ui.updatePrivateRadio->setChecked(true);
        setPrivatePlaylist();
    }

    setChecked(ui.fullscreenChk, "main/fullscreen", true);
    setChecked(ui.screenSaverChk, "main/no_screensaver", true);
    setChecked(ui.fixedscreenChk, "main/fixedscreen", true);
    setChecked(ui.resetChk, "main/reset", true);

    setText(ui.largeur_edit, "main/largeur_edit");
    setText(ui.hauteur_edit, "main/hauteur_edit");


    if (! settings->get("main/admin_passwd").toString().isEmpty())
        ui.passwordChk->setChecked(true);

    QString fontStr(settings->get("webpage/default_font").toString());
    QFont font; font.fromString(fontStr);
    ui.fontBtn->setText(
                QString("%1 (%2)")
                .arg(font.family()).arg(font.pointSize()));

    // webkiosk
    setChecked(ui.hideToolbarChk, "main/hide_toolbar", true);
    setChecked(ui.browseFSChk, "main/can_access_fs", true);
    setChecked(ui.displayClearChk, "main/can_end_session", true);
    setChecked(ui.printChk, "main/can_print", true);
    setChecked(ui.sessionAtStartupChk, "main/start_with_session", true);
    setText(ui.homeEdit, "main/default_url");

    setSpin(ui.sessionTimeoutSpin, "main/stop_session_after_seconds");

    setChecked(ui.sessionTimeChk, "main/with_timer", true);
    setSpin(ui.sessionTimeSpin, "main/default_timer");
    setText(ui.postSessionUrlEdit, "main/post_session_uri");
    setSpin(ui.postSessionTimeSpin, "main/post_session_uri_time");
    setZoom(ui.zoomFactor, "main/zoomFactor");
    setChecked(ui.externalBrowserChk, "main/external_browser", true);

    QString lmode = settings->get("main/list_mode").toString();
    int lindex = 0;
    if (lmode == "whitelist")
        lindex = 2;
    else if (lmode == "blacklist")
        lindex = 1;
    ui.restrictionsCombo->setCurrentIndex(lindex);
    setChecked(ui.regexChk, "main/list_use_regex", true);
    QString f(settings->whileListFilename());
    restrictionsLoadFromFile(f);
    genRestrictionsList();

    // display
    if(settings->get("slideshow/resize").toBool())
        ui.resizeRadio->setChecked(true);
    else if(settings->get("slideshow/auto_scroll").toBool())
        ui.scrollRadio->setChecked(true);
    else if(settings->get("slideshow/stretch").toBool())
        ui.stretchRadio->setChecked(true);
    else
        ui.noneRadio->setChecked(true);

    //general display
    setChecked(ui.hideMouseChk, "slideshow/hide_pointer", true);
    setChecked(ui.muteChk, "slideshow/mute", true);
    setSpin(ui.displayTimeSpin, "slideshow/default_time");
    setChecked(ui.usbAuto, "slideshow/usb_auto", true);
    setChecked(ui.shuffle, "slideshow/shuffle", true);

    setChecked(ui.remoteChk,"remote/enabled",true);
    ui.directChk->setEnabled(ui.remoteChk->isChecked());
    setChecked(ui.directChk,"slideshow/direct",true);
    on_privateServer_toggled(ui.privateServer->isChecked());
    setSpin(ui.refreshPlaylistSpin, "slideshow/refresh_after_minutes");
    setChecked(ui.externalPlayerChk,"slideshow/external_player",true);
    setChecked(ui.slideshowExternalBrowserChk,"slideshow/external_browser", true);
    playlistLoadFromFile(settings->playlistFilename());
    genPlaylistList();
    connect(m_localPlaylistView->selectionModel(),
            SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)),
            this,
            SLOT(on_PlaylistSelectionChanged(const QItemSelection &, const QItemSelection &)));

    // network
    switch (settings->get("network/proxy_type").toInt())
    {
        case 1:
            ui.proxyAutoRadio->setChecked(true);
            break;
        case 2:
            ui.proxyManualRadio->setChecked(true);
            break;
        default:
            ui.proxyNoRadio->setChecked(true);
            break;
    }

    setText(ui.proxyUrlEdit, "network/proxy_uri");
    setText(ui.proxyUserEdit, "network/proxy_user");
    setText(ui.proxyPasswordEdit, "network/proxy_passwd");
    setText(ui.invoice,"slideshow/invoice");
    setText(ui.mail,"slideshow/email");
    setText(ui.newTabUrl,"main/new_tab_url");

    setChecked(ui.tvChk, "remote/tv", true);
    this->ui.tvChk->setEnabled(this->ui.remoteChk->isChecked());
    setChecked(ui.localplaylistChk, "remote/localplaylist", true);
    this->ui.localplaylistChk->setEnabled(this->ui.remoteChk->isChecked());
    setChecked(ui.localfsChk, "remote/localfs", true);
    this->ui.localfsChk->setEnabled(this->ui.remoteChk->isChecked());
}

void SettingsDialog::playlistLoadFromFile(QString filename)
{
    QFile f(filename);
    if (! f.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        qDebug() << "ERROR: Failed to read playlist file " << filename << " !";
        return;
    }

    QTextStream in(&f);
    while (!in.atEnd())
    {
        QString line = in.readLine();
        QRegExp rx("^([^#].*)\\s+(\\d+)\\s*$");
        rx.setMinimal(true);
        if (rx.exactMatch(line))
        {
            QStringList list = rx.capturedTexts();
            m_localPlaylistView->add(list.at(1), list.at(2).toInt());
        }
    }
    f.close();
}

QString cleanJsonString(QString data)
{
    // Remove inline comms
    QRegExp pattern = QRegExp("(^|\\[|\\{|,|\\n|\\s)//.*($|\\n)");
    pattern.setMinimal(true); //ungreedy
    data.replace(pattern, "\\1\n");
    data.replace(pattern, "\\1\n"); //2 times, I am not sure why...
    // Remove bloc comms
    pattern = QRegExp("/\\*.*\\*/");
    pattern.setMinimal(true); //ungreedy
    data.replace(pattern, "");
    // Add first and last brace
    if (!data.startsWith("{")) {
        data = "{\n"+data;
    }
    if (!data.endsWith("}")) {
        data += "\n}";
    }
    // Remove commas before } or ]
    pattern = QRegExp(",(\\s*[}\\]])");
    pattern.setMinimal(true); //non-greedy
    data.replace(pattern, "\\1");
    return data;
}

void SettingsDialog::loadChannels(void)
{
    qDebug() << "loadChannels";

    // Set proxy settings.
    ProxyFactory* factory = new ProxyFactory(settings->getCommandLineParser());
    QNetworkProxyFactory::setApplicationProxyFactory(factory);

    // Remove the previous items.
    ui.listWidget->clear();

    qDebug() << getSlideBaseUrl();

    // Retrieve data from server.
    {
        QUrl url;            // Server location.
        QByteArray postData; // Data to send for authentication.

        if (settings->get("slideshow/invoice").toString().isEmpty()
           || settings->get("slideshow/email").toString().isEmpty())
        {
            url = QUrl(getSlideBaseUrl() + "/webapp/db/channels");
            qDebug() << "after set url";
        }
        else
        {
            url = QUrl(getSlideBaseUrl() + "/webapp/api/user/" +
                       settings->get("slideshow/invoice").toString());
            postData.append("model=");

            // Javascript engine (overkill ?).
            QScriptEngine engine;

            QScriptValue v = engine.newObject();
            v.setProperty("invoice",
                          settings->get("slideshow/invoice").toString());
            v.setProperty("email",
                          settings->get("slideshow/email").toString());
            engine.evaluate("function toString() "
                            "{ return JSON.stringify(this) }");

            QScriptValue toString = engine.globalObject().property("toString");
            postData.append(toString.call(v).toString());
            qDebug() << postData;
        }

        // Create request.
        QNetworkRequest req;
        {
            // Set url.
            req.setUrl(url);

            // Add content type header.
            req.setHeader(QNetworkRequest::ContentTypeHeader,
                          "application/x-www-form-urlencoded; charset=UTF-8");

            // Add cookie header.
            {
                QNetworkCookie cookie;
                cookie.setName("lang");
                cookie.setValue("fr");

                QList<QNetworkCookie> cookies;
                cookies.append(cookie);

                QVariant var;
                var.setValue(cookies);

                req.setHeader(QNetworkRequest::CookieHeader, var);
            }
        }

        // Send request to server.
        QNetworkAccessManager nam;
        QNetworkReply* rep = nam.post(req,postData);

        // Wait until server reply is completed.
        {
            QEventLoop loop;
            connect(rep, SIGNAL(finished()), &loop,SLOT(quit()));
            loop.exec();
            qDebug() << "AfterLoop" << rep->errorString();
        }

        if (rep->error() == QNetworkReply::NoError)
        {
            QScriptEngine engine;
            QString re(rep->readAll());
            QScriptValue result = engine.evaluate("(" + re.toUtf8() + ")");
            QScriptValueIterator it(
                        result.property("screens").isValid()
                        ? result.property("screens")
                        : result);
            it.setValue(result.property("screens").isValid()
                        ? result.property("screens")
                        : result);

            //qDebug() << result.toVariant();

            // Extract playlist elements.
            QListWidgetItem* current = NULL;
            QString activeChannel =
                    settings->get("slideshow/channel").toString();
            qDebug() << "Getting channel:" << activeChannel;
            int i = 0;
            while (it.hasNext())
            {
                it.next();
                QString label = it.value().property("label").toString();
                if (label.isEmpty()) continue;

                label.prepend(QString::number(i) + " - ");
                i++;

                QListWidgetItem* item = new QListWidgetItem(label);
                if (!current || activeChannel == label)
                    current = item;
                item->setData(32, "/webapp/p/" +
                              it.value().property("login").toString()  + "-" +
                              it.value().property("screen").toString() + "/" +
                              (it.value().property("token").isValid()
                               ? it.value().property("token").toString()
                               : "public"));
                ui.listWidget->addItem(item);
            }

            if (current)
            {
                qDebug() << "Setting:"
                         << current->text()
                         << " as current item.";
                ui.listWidget->setCurrentItem(current);
            }

            qDebug() << ui.listWidget->count();

            // Select the first item as default one.
            if (ui.listWidget->count())
            {
                QString pages = settings->get("slideshow/pages").toString();
                bool have_to_set = true;
                for (int i = 0; i < ui.listWidget->count(); ++i)
                {
                    QString current = getSlideBaseUrl() +
                        ui.listWidget->item(i)->data(32).toString();

                    qDebug() << "Current:" << current;
                    qDebug() << "Loaded:" << pages;
                    if (current == pages)
                    {
                        have_to_set = false;
                        qDebug() << "Don't have to set pages...";
                        break;
                    }
                }

                if (have_to_set)
                {
                    QString pages = getSlideBaseUrl() +
                        ui.listWidget->item(0)->data(32).toString();
                    settings->set("slideshow/pages", pages);
                    settings->set("slideshow/channel", current->text());

                    qDebug() << "Setting pages:" << pages;
                }
            }
        }
        else
        {
            QString reply = rep->readAll();
            qDebug() << "Network request error: " << rep->errorString();
            qDebug() << "Reply: \n" << reply;
            if(reply.contains("Authentication failed"))
            {
                ui.channelInfo->setText("Authentification failed");
            }
            else if(reply == "")
            {
                ui.channelInfo->setText("No network");
            }
        }
    }
}

void SettingsDialog::loadPlaylist() {}

QString SettingsDialog::getSlideBaseUrl() const
{
    QString mode = settings->get("slideshow/playlist_update_mode").toString();
    qDebug() << "update mode: " << mode;
    return (mode == "serverLinutopTVPrivate")
            ? settings->get("slideshow/url").toString()
            : "http://linutop.tv";
}

void SettingsDialog::genPlaylistList()
{
    QStringList l;
    QStandardItemModel *m =
            static_cast<QStandardItemModel *>(m_localPlaylistView->model());

    for (int row=0; row < m->rowCount(); ++row)
    {
        QString uri = m->item(row, 0)->data(Qt::DisplayRole).toString();
        int time = m->item(row, 1)->data(Qt::DisplayRole).toInt();
        l << QString("%1 %2").arg(uri).arg(time);
    }

    settings->playlistItems = l;
}

bool SettingsDialog::restrictionsLoadFromFile(QString &filename)
{
    ui.restrictionsListView->clear();
    QFile f(filename);
    if (! f.open(QIODevice::ReadOnly | QIODevice::Text))
        return false;

    QTextStream in(&f);
    QStringList l;
    while (!in.atEnd())
    {
        QString line = in.readLine();
        if (line.isEmpty())
            continue;
        ui.restrictionsListView->addItem(line);
        l << line;
    }
    f.close();
    settings->restrictions = l;

    return true;
}

bool SettingsDialog::restrictionsSaveToFile(QString &filename)
{
    QFile f(filename);
    if (! f.open(QIODevice::WriteOnly | QIODevice::Text))
        return false;

    QTextStream out(&f);
    for (int i=0; i<ui.restrictionsListView->count(); ++i)
        out << ui.restrictionsListView->item(i)->text() << "\n";
    f.close();

    return true;
}

void SettingsDialog::on_restrictionsSelection_changed(
        const QItemSelection &selected,
        const QItemSelection &deselected)
{
    bool b = ui.restrictionsListView->selectionModel()->hasSelection();
    ui.restrictionDeleteBtn->setEnabled(b);
}

void SettingsDialog::showError(QNetworkReply::NetworkError error)
{
    qDebug()<<error;
}

void SettingsDialog::on_helpBtn_clicked()
{
    QProcess *p = new QProcess(this);
    p->start("yelp /usr/share/gnome/help/linutop-webkiosk/C/webkiosk.xml");
}

void SettingsDialog::on_testFinished(
        int exitCode, QProcess::ExitStatus exitStatus)
{
    QProcess p;
    p.start("rm -rf /tmp/linutop-kiosk-test");
    p.waitForFinished();
    delete m_testProcess;
    m_testProcess = 0;
}


void SettingsDialog::exit()
{
    //Récupération de l'id d'une fenêtre de test
    QProcess id_linutopkiosk;
    id_linutopkiosk.start("sh", QStringList()<<"-c"<<"ps -faux | grep \"linutop-kiosk --basepath /tmp\" | grep root |  awk '{print $2}'| head -1");
    id_linutopkiosk.waitForFinished();

    QByteArray output = id_linutopkiosk.readAllStandardOutput();
    qDebug()<<"output :" + output;

    //Fermeture de la fenêtre
    QProcess cmdKill;
    cmdKill.start("sh", QStringList()<<"-c"<< "sudo kill -9 " + output);
    cmdKill.waitForFinished();

}

void SettingsDialog::creer()
{
    QWidget *mv = new QWidget;
    QPushButton *button = new QPushButton("Escape to quit",mv);

    mv->setGeometry(0,0,100,50);
    button->resize(100,50);
    mv->setWindowFlags(Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint);
    mv->show();


    connect(button, SIGNAL(clicked()), this, SLOT(exit()));
    connect(button, SIGNAL(clicked()), mv, SLOT(close()));

    mv->deleteLater();
}

void SettingsDialog::on_testBtn_clicked()
{

    exit();
    QDir d("/tmp/linutop-kiosk-test");
    d.mkpath("/tmp/linutop-kiosk-test");
    settings->save(true, "/tmp/linutop-kiosk-test");
    m_testProcess = new QProcess(this);
    QStringList args;
    args << CommandLineParser::basePathArgument()
    << "/tmp/linutop-kiosk-test";
    #ifndef NDEBUG
    QString prog = "./linutop-kiosk";
    #else
    QString prog = "linutop-kiosk";
    #endif // DEBUG
    qDebug() << "Running:" << prog.toStdString().c_str()
    << args.join(" ").toStdString().c_str();




ProcessOutputReader* reader = new ProcessOutputReader(m_testProcess);
    m_testProcess->start(prog, args);
    reader->loop();


}

void SettingsDialog::on_cancelBtn_clicked() { close(); }

void SettingsDialog::on_okBtn_clicked()
{
    bool canClose = true;

    if (! settings->get("main/reset").toBool()){
    if (! ui.passwordChk->isChecked())
        settings->set("main/admin_passwd", QString(""));
    if (ui.updateOnlineRadio->isChecked() &&
            settings->get("slideshow/pages").toString().isEmpty())
    {
        qDebug() << QMessageBox::warning(
                        this,
                        tr("No channel"),
                        tr("No Channel has been selected, "
                           "switching back to local playlist"),
                        tr("Close anyway"),
                        tr("Go back"));
        settings->set("slideshow/playlist_update_mode",
                      QString("localPlaylist"));
        ui.updatePlaylistRadio->setChecked(true);
        canClose = false;
    }
    else if (ui.updatePlaylistRadio->isChecked() &&
             m_localPlaylistView->isEmpty())
    {
        canClose = false;
        QMessageBox::warning(
                    this,
                    tr("Empty playlist"),
                    tr("Your local playlist is empty, please add an item"));
    }
    if(settings->get("main/fixedscreen").toBool()){
        QRegExp Exp ("^[0-9]*$");
        if (ui.largeur_edit->text() == "" || ui.hauteur_edit->text() ==""){
              canClose = false;
              QMessageBox::warning(
                           this,
                           tr("Empty LED"),
                           tr("width or height value for the LED panel in General is empty, please enter a number"));
        }
        else if(!Exp.exactMatch(ui.largeur_edit->text()) || !Exp.exactMatch(ui.hauteur_edit->text())) {
            canClose = false;
            QMessageBox::warning(
                        this,
                        tr("Error LED"),
                        tr("width or height value in General contain a character, please adjust value"));
        }
    }

    if (!newTemplate.isEmpty())
    {
        QFile fi(newTemplate);
        QFile::remove("/usr/share/linutop-webkiosk/text.html");
        fi.copy("/usr/share/linutop-webkiosk/text.html");
    }
    }
    if(canClose)
    {
        settings->save();
        close();
    }
}

/* main */

void SettingsDialog::on_useBtnGrp_buttonClicked(QAbstractButton *btn)
{
    QTabBar *tb = ui.tabWidget->tabBar();
    if (btn == ui.useKioskRadio)
    {
        tb->setTabEnabled(1, true);
        tb->setTabEnabled(2, false);
        settings->set("main/mode", QString("kiosk"));
        settings->set("main/after_web_session", QString("session"));
        settings->set("main/start_with_session", true);
    }
    else if (btn == ui.useDisplayChk)
    {
        tb->setTabEnabled(1, false);
        tb->setTabEnabled(2, true);
        settings->set("main/mode", QString("display"));
    }
    else
    {
        tb->setTabEnabled(1, true);
        tb->setTabEnabled(2, true);
        settings->set("main/mode", QString("kiosk"));
        settings->set("main/after_web_session", QString("slideshow"));
        if (settings->get("main/with_timer").toBool())
            settings->set("main/start_with_session", false);
    }
}

void SettingsDialog::on_fullscreenChk_toggled(bool checked)
{
    settings->set("main/fullscreen", checked);
}

//définition de la méthode onfixedscreenChk
void SettingsDialog::on_fixedscreenChk_toggled(bool checked)
{
    settings->set("main/fixedscreen", checked);
}

void SettingsDialog::on_resetChk_toggled(bool checked)
{
    qDebug () << "nb reset : " << nbreset;
    if(nbreset == 0){
        nbreset++;
        qDebug() << "on rentre dans la variable de reset";
        settings->set("main/reset", checked);
        settings->populateDefaults();
        playlistLoadFromFile(CommandLineParser::defaultIniPlaylistFilename());
    }
}

void SettingsDialog::on_screenSaverChk_toggled(bool checked)
{
    settings->set("main/no_screensaver", checked);
}

void SettingsDialog::on_startupChk_toggled(bool checked)
{
    settings->setAutostarted(checked);
}

void SettingsDialog::on_passwordChk_toggled(bool checked)
{
    ui.passwordBtn->setEnabled(checked);
}

void randomString(char *s, const int len) {
    static const char alphanum[] =
        "0123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz";

    for (int i = 0; i < len; ++i) {
        s[i] = alphanum[rand() % (sizeof(alphanum) - 1)];
    }

    s[len] = 0;
}

void SettingsDialog::on_passwordBtn_clicked()
{
    PasswordDialog d(this);
    if (d.exec() == QDialog::Accepted)
    {
        char salt[10] = "$1$";
        randomString(salt+3, 6);
        char *c = crypt(d.password().toUtf8().data(), salt);
        settings->set("main/admin_passwd", QString(c));
    }
    d.deleteLater();
}

void SettingsDialog::on_fontBtn_clicked()
{
    bool ok;
    QFont font;
    {
        QString fontStr = settings->get("webpage/default_font").toString();
        if (!fontStr.isEmpty())
            font.fromString(fontStr);
        font = QFontDialog::getFont(&ok, font, this);
    }

    if (ok)
    {
        settings->set("webpage/default_font", font.toString());
        ui.fontBtn->setText(QString("%1 (%2)")
                            .arg(font.family()).arg(font.pointSize()));
    }
}

/* kiosk */
void SettingsDialog::on_hideToolbarChk_toggled(bool checked)
{
    settings->set("main/hide_toolbar", checked);
}

void SettingsDialog::on_browseFSChk_toggled(bool checked)
{
    settings->set("main/can_access_fs", checked);
}

void SettingsDialog::on_displayClearChk_toggled(bool checked)
{
    settings->set("main/can_end_session", checked);
}

void SettingsDialog::on_printChk_toggled(bool checked)
{
    settings->set("main/can_print", checked);
}

void SettingsDialog::on_sessionAtStartupChk_toggled(bool checked)
{
    settings->set("main/start_with_session", checked);
}

void SettingsDialog::on_homeEdit_textChanged(QString value)
{
    settings->set("main/default_url", value);
}



void SettingsDialog::on_sessionTimeoutSpin_valueChanged(int value)
{
    settings->set("main/stop_session_after_seconds", value);
}

void SettingsDialog::on_externalBrowserChk_toggled(bool checked)
{
    settings->set("main/external_browser", checked);
    if(checked)
    {
        ui.groupBox_9->setDisabled(true);
        ui.newTabUrl->setDisabled(true);
        ui.sessionTimeoutSpin->setDisabled(true);
        ui.browseFSChk->setDisabled(true);
        ui.displayClearChk->setDisabled(true);
        ui.hideToolbarChk->setDisabled(true);
        ui.printChk->setDisabled(true);
        ui.sessionAtStartupChk->setDisabled(true);
        ui.postSessionTimeSpin->setDisabled(true);
        ui.postSessionUrlEdit->setDisabled(true);
        ui.label->setDisabled(true);
        ui.fontBtn->setDisabled(true);
        ui.zoomFactor->setDisabled(true);
        ui.sessionTimeChk->setDisabled(true);
        ui.sessionTimeSpin->setDisabled(true);
        ui.label_3->setDisabled(true);
        ui.label_14->setDisabled(true);
        ui.label_23->setDisabled(true);
        ui.label_25->setDisabled(true);
        ui.label_26->setDisabled(true);
        ui.label_27->setDisabled(true);
        ui.label_28->setDisabled(true);
        ui.label_30->setDisabled(true);
    }
    else
    {
        ui.groupBox_9->setDisabled(false);
        ui.newTabUrl->setDisabled(false);
        ui.sessionTimeoutSpin->setDisabled(false);
        ui.browseFSChk->setDisabled(false);
        ui.displayClearChk->setDisabled(false);
        ui.hideToolbarChk->setDisabled(false);
        ui.printChk->setDisabled(false);
        ui.sessionAtStartupChk->setDisabled(false);
        ui.postSessionTimeSpin->setDisabled(false);
        ui.postSessionUrlEdit->setDisabled(false);
        ui.label->setDisabled(false);
        ui.fontBtn->setDisabled(false);
        ui.zoomFactor->setDisabled(false);
        ui.sessionTimeChk->setDisabled(false);
        ui.sessionTimeSpin->setDisabled(false);
        ui.label_3->setDisabled(false);
        ui.label_14->setDisabled(false);
        ui.label_23->setDisabled(false);
        ui.label_25->setDisabled(false);
        ui.label_26->setDisabled(false);
        ui.label_27->setDisabled(false);
        ui.label_28->setDisabled(false);
        ui.label_30->setDisabled(false);
    }
}

void SettingsDialog::on_sessionTimeChk_toggled(bool checked)
{
    settings->set("main/with_timer", checked);
    settings->set("slideshow/stop_on_event", checked);
}

void SettingsDialog::on_sessionTimeSpin_valueChanged(int value)
{
    settings->set("main/default_timer", value);
}

void SettingsDialog::on_postSessionUrlEdit_textEdited(QString value)
{
    settings->set("main/post_session_uri", value);
}

void SettingsDialog::on_postSessionTimeSpin_valueChanged(int value)
{
    settings->set("main/post_session_uri_time", value);
}

void SettingsDialog::on_zoomFactor_valueChanged(double value)
{
    settings->set("main/zoomFactor", value);
}

void SettingsDialog::on_restrictionsCombo_currentIndexChanged(int value)
{
    QString s;
    switch (value)
    {
        case 0: s = QString ("none"); break;
        case 1: s = QString ("blacklist"); break;
        case 2: s = QString ("whitelist"); break;
    }
    settings->set("main/list_mode", s);
}

void SettingsDialog::on_regexChk_toggled(bool checked)
{
    settings->set("main/list_use_regex", checked);
}

void SettingsDialog::genRestrictionsList()
{
    QStringList l;

    for (int i=0; i < ui.restrictionsListView->count(); ++i)
        l << ui.restrictionsListView->item(i)->data(Qt::DisplayRole).toString();

    settings->restrictions = l;
}

void SettingsDialog::on_restrictionAddBtn_clicked()
{
    AddUrlDialog d;
    int r = d.exec();
    QString url = d.url();
    if ((r == QDialog::Accepted) && (! url.isEmpty()))
            ui.restrictionsListView->addItem(url);

    genRestrictionsList();
}

void SettingsDialog::on_restrictionDeleteBtn_clicked()
{
    ui.restrictionsListView->setUpdatesEnabled(false);
    QModelIndex idx = ui.restrictionsListView
            ->selectionModel()->selectedIndexes().at(0);
    ui.restrictionsListView->model()->removeRow(idx.row());
    ui.restrictionsListView->setUpdatesEnabled(true);
    genRestrictionsList();
}

void SettingsDialog::on_restrictionImportBtn_clicked()
{
    QString filename = QFileDialog::getOpenFileName(this, "Import");
    if (! filename.isEmpty())
        restrictionsLoadFromFile(filename);
}

void SettingsDialog::on_restrictionExportBtn_clicked()
{
    QString filename = QFileDialog::getOpenFileName(this, "Export");
    if (! filename.isEmpty())
        restrictionsSaveToFile(filename);
}

/* display */

void SettingsDialog::on_hideMouseChk_toggled(bool checked)
{
    settings->set("slideshow/hide_pointer", checked);
}

void SettingsDialog::on_muteChk_toggled(bool checked)
{
    settings->set("slideshow/mute", checked);
}

void SettingsDialog::on_displayTimeSpin_valueChanged(int value)
{
    settings->set("slideshow/default_time", value);
}

void SettingsDialog::on_shuffle_toggled(bool checked)
{
    settings->set("slideshow/shuffle", checked);
}


void SettingsDialog::on_playlistBtnGrp_buttonClicked(QAbstractButton *btn)
{
    bool online = (btn == ui.updateOnlineRadio);
    bool local  = (btn == ui.updatePlaylistRadio);
    bool priv   = (btn == ui.updatePrivateRadio);

    radioChoiceInit();

    if (online) setLinutopTV(ui.privateServer->isChecked());
    if (priv)   setPrivatePlaylist();
    if (local)  setLocalPlaylist();
}

void SettingsDialog::on_playlistUsbUpdate_toggled(bool checked)
{
    settings->set("slidehow/do_local_update", checked);
}

void SettingsDialog::on_remoteEdit_textChanged(QString value)
{
    if (ui.privateServer->isChecked())
    {
        if(!value.startsWith("http://"))
            value.prepend("http://");
        settings->set("slideshow/url", value);
        loadChannels();
    }
}

void SettingsDialog::on_privateEdit_textChanged(QString value)
{
    settings->set("slideshow/pages", value);
    loadPlaylist();
}

void SettingsDialog::on_refreshPlaylistSpin_valueChanged(int value)
{
    settings->set("slideshow/refresh_after_minutes", value);
}

void SettingsDialog::on_localPlaylistDeleteBtn_clicked()
{
    // Need to uncheck usb-auto ?
    if (ui.usbAuto->isChecked())
    {
        QStandardItemModel *m =
                static_cast<QStandardItemModel *>(m_localPlaylistView->model());
        int cur = m_localPlaylistView->currentIndex().row();
        if (m->item(cur, 0)->data(Qt::DisplayRole).toString() == "usb-auto")
            ui.usbAuto->setChecked(false);
    }

    m_localPlaylistView->delSelection();
    genPlaylistList();
}

void SettingsDialog::on_localPlaylistUpBtn_clicked()
{
    m_localPlaylistView->moveSelection(-1);
    genPlaylistList();
}

void SettingsDialog::on_localPlaylistDownBtn_clicked()
{
    m_localPlaylistView->moveSelection(1);
    genPlaylistList();
}

void SettingsDialog::on_localPlaylistScreenOffBtn_clicked()
{
    QString string = "Screen OFF - From: 20:00  To: 08:00";
    m_localPlaylistView->add(string, 0);
}
void SettingsDialog::on_localPlaylistAddBtn_clicked()
{
    AddPlaylistItemDialog d(
                settings->get("slideshow/default_time").toInt(), this);
    if (d.exec() != QDialog::Accepted) return;

    QString uri = d.text();
    uri = uri.trimmed();
    if (uri.isEmpty()) return;

    if (uri.at(0) == '/')
        uri = QString("file://%1").arg(uri);
    QMimeDatabase db;
    QMimeType mimetype = db.mimeTypeForUrl(uri);

    int time = d.time();
    if (mimetype.name().startsWith("video/") || mimetype.name().startsWith("audio/"))
        time = 0;

    // Remove previous usb-auto.
    if (uri == "usb-auto")
    {
        QStandardItemModel *m =
                static_cast<QStandardItemModel *>(m_localPlaylistView->model());

        for (int row = 0; row < m->rowCount(); ++row)
        {
            QString uri = m->item(row, 0)->data(Qt::DisplayRole).toString();
            if (uri == "usb-auto")
                m_localPlaylistView->model()->removeRow(row);
        }
    }

    m_localPlaylistView->add(uri, time);

    if (uri == "usb-auto")
        ui.usbAuto->setChecked(true);

    genPlaylistList();
}

void SettingsDialog::on_localPlaylistCookiesBtn_clicked()
{
    QStringList urls;
    QStandardItemModel *m =
            static_cast<QStandardItemModel*>(m_localPlaylistView->model());

    m->rowCount();

    for (int row=0; row < m->rowCount(); ++row)
    {
        QString url = m->item(row, 0)->data(Qt::DisplayRole).toString();
        if (url.startsWith("http://") || url.startsWith("https://"))
            urls << url;
    }
    if (urls.empty())
    {
        QMessageBox::warning(
            this,
            "Warning",
            tr("No website address in the list.\n"
               "Please add one before using this button."));
        return;
    }
    CookieSaver cs(&urls, this);
    if (cs.exec() == QDialog::Accepted)
    {
        qDebug() << "Accepted";
        settings->cookiesList = cs.allCookies();
    }
}

void SettingsDialog::on_usbAuto_toggled(bool value)
{
    if (m_isInit) return;

    qDebug() << "USB Auto clicked:" << value;
    settings->set("slideshow/usb_auto", value);

    if (value)
    {
        // Find "usb-auto" to know if need to add it.
        QStandardItemModel *m =
                static_cast<QStandardItemModel *>(m_localPlaylistView->model());

        bool to_add = true;
        for (int row = 0; row < m->rowCount(); ++row)
        {
            QString uri = m->item(row, 0)->data(Qt::DisplayRole).toString();
            if (uri == "usb-auto")
            {
                to_add = false;
                break;
            }
        }

        if (to_add)
        {
            int time = settings->get("slideshow/default_time").toInt();
            m_localPlaylistView->add("usb-auto", time);
            genPlaylistList();
        }
    }
    else
    {
        // Find "usb-auto" and remove it.
        QStandardItemModel *m =
                static_cast<QStandardItemModel *>(m_localPlaylistView->model());

        for (int row = 0; row < m->rowCount(); ++row)
        {
            QString uri = m->item(row, 0)->data(Qt::DisplayRole).toString();
            if (uri == "usb-auto")
                m_localPlaylistView->model()->removeRow(row);
        }

        genPlaylistList();
    }
}

void SettingsDialog::on_PlaylistSelectionChanged(
        const QItemSelection &selected,
        const QItemSelection &unselected)
{
    bool b = m_localPlaylistView->selectionModel()->hasSelection();
    ui.localPlaylistDeleteBtn->setEnabled(b);
    ui.localPlaylistUpBtn->setEnabled(b);
    ui.localPlaylistDownBtn->setEnabled(b);
}

/* network */
void SettingsDialog::on_proxyBtnGrp_buttonClicked(QAbstractButton *btn)
{
    int t;
    bool b;

    if (btn == ui.proxyAutoRadio)
    {
        t = 1;
        b = false;
    }
    else if (btn == ui.proxyManualRadio)
    {
        t = 2;
        b = true;
    }
    else
    {
        t = 0;
        b = false;
    }

    settings->set("network/proxy_type", t);
    ui.proxyUrlEdit->setEnabled(b);
    ui.proxyUserEdit->setEnabled(b);
    ui.proxyPasswordEdit->setEnabled(b);
}

void SettingsDialog::on_proxyUserEdit_textChanged(QString s)
{ settings->set("network/proxy_user", s); }

void SettingsDialog::on_largeur_edit_textChanged(QString s)
{ settings->set("main/largeur_edit", s); }

void SettingsDialog::on_hauteur_edit_textChanged(QString s)
{ settings->set("main/hauteur_edit", s); }


void SettingsDialog::on_proxyPasswordEdit_textChanged(QString s)
{ settings->set("network/proxy_passwd", s); }

void SettingsDialog::on_proxyUrlEdit_textChanged(QString s)
{ if(!s.startsWith("http://")) s.prepend("http://");
    settings->set("network/proxy_uri", s);}

void SettingsDialog::on_invoice_textChanged(QString s)
{
    if (s.isEmpty()) updateChannelInfo(true);
    else updateChannelInfo(
                settings->get("slideshow/email").toString().isEmpty());
    settings->set("slideshow/invoice", s);
}

void SettingsDialog::on_mail_textChanged(QString s)
{
    if (s.isEmpty()) updateChannelInfo(true);
    else updateChannelInfo(
                settings->get("slideshow/invoice").toString().isEmpty());
    settings->set("slideshow/email", s);
}

void SettingsDialog::on_connect_clicked()
{
    settings->set("slideshow/invoice",ui.invoice->text());
    settings->set("slideshow/email",ui.mail->text());
    updateChannelInfo(
                    settings->get("slideshow/invoice").toString().isEmpty());
    loadChannels();
}


void SettingsDialog::on_listWidget_itemClicked(QListWidgetItem *item)
{
    QString url = getSlideBaseUrl();
    QString pages = url + item->data(32).toString();
    QString channel = item->text();
    qDebug() << "Setting pages:" << pages;
    qDebug() << "Setting channel:" << channel;
    settings->set("slideshow/pages", pages);
    settings->set("slideshow/channel", channel);



}

void SettingsDialog::on_remoteChk_toggled(bool checked)
{
    settings->set("remote/enabled",checked);
    ui.directChk->setEnabled(checked);
    ui.tvChk->setEnabled(checked);
    ui.localplaylistChk->setEnabled(checked);
    ui.localfsChk->setEnabled(checked);
    if(ui.remoteChk->isChecked()){
        QString ip;
        QStringList addresses;
        foreach(QHostAddress address, QNetworkInterface::allAddresses())
        {
            // Filtre les adresses localhost ...
            if(address != QHostAddress::LocalHostIPv6
                    && address != QHostAddress::LocalHost
                    // ... APIPA ...
                    && !address.isInSubnet(QHostAddress::parseSubnet("169.254.0.0/16"))
                    // ...  Local IPv6 link
                    && !address.isInSubnet(QHostAddress::parseSubnet("FE80::/64"))
                    && address.protocol() == QAbstractSocket::IPv4Protocol) //disable ipv6
            {
                addresses << address.toString();
            }
        }


        foreach(QString address, addresses)
        {
            qDebug() << address;
            ip = address;
        }

        if (ip.isEmpty()) ip = "127.0.0.1";

        ip = "http://" + ip + ":" + settings->get("remote/port").toString();
        QString href = ip;
        QString key = settings->get("remote/key").toString();
        if (key.size() > 0)
            href += "?k=" + key;
        ui.remoteLabel->setText("<a href='" + href + "'>" + href + "</a>");

        settings->set("remote/url", ip);
    }
    else{
        settings->set("remote/url", "");
        ui.remoteLabel->setText("");
    }
}

void SettingsDialog::setLinutopTV(bool priv)
{
    ui.invoice->setEnabled(true);
    ui.mail->setEnabled(true);
    ui.connect->setEnabled(true);

    ui.remoteEdit->setEnabled(priv);
    ui.refreshPlaylistSpin->setEnabled(true);
    ui.listWidget->setEnabled(true);
    if (priv)
        settings->set("slideshow/playlist_update_mode",
                      QString("serverLinutopTVPrivate"));
    else
        settings->set("slideshow/playlist_update_mode",
                      QString("serverLinutopTV"));

    if (!priv) ui.remoteEdit->setText("http://www.linutop.tv");
    else setText(ui.remoteEdit, "slideshow/url");

    loadChannels();
}

void SettingsDialog::setPrivatePlaylist(void)
{
    ui.invoice->setDisabled(true);
    ui.mail->setDisabled(true);
    ui.connect->setDisabled(true);

    ui.privateEdit->setEnabled(true);
    ui.refreshPlaylistSpin->setEnabled(true);
    settings->set("slideshow/playlist_update_mode",
                  QString("serverPlaylistPrivate"));
    loadPlaylist();
}

void SettingsDialog::setLocalPlaylist(void)
{
    ui.invoice->setDisabled(true);
    ui.mail->setDisabled(true);
    ui.connect->setDisabled(true);

    m_localPlaylistView->setEnabled(true);
    ui.usbAuto->setEnabled(true);
    ui.shuffle->setEnabled(true);
    settings->set("slideshow/playlist_update_mode",
                  QString("localPlaylist"));
    loadPlaylist();
}

void SettingsDialog::radioChoiceInit(void)
{
    ui.usbAuto->setEnabled(false);
    ui.shuffle->setEnabled(false);
    ui.privateEdit->setEnabled(false);
    ui.remoteEdit->setEnabled(false);
    ui.refreshPlaylistSpin->setEnabled(false);
    ui.listWidget->setEnabled(false);
    m_localPlaylistView->setEnabled(false);
}

void SettingsDialog::updateChannelInfo(bool publ)
{
    ui.channelInfo->setText(
                publ
                ? tr("Channel (public)")
                : tr("Channel (private)"));
}


void SettingsDialog::on_privateServer_toggled(bool checked)
{
    if (!m_isInit)
    {
        radioChoiceInit();
        ui.updateOnlineRadio->setChecked(true);
        setLinutopTV(checked);
    }
}

void SettingsDialog::on_remotepassword_textChanged(QString s)
{
    QString key = s;
    settings->set("remote/key", key);
    this->on_remoteChk_toggled(settings->get("remote/enabled").toBool());
}




void SettingsDialog::on_directChk_toggled(bool checked)
{
    settings->set("slideshow/direct",checked);
}

void SettingsDialog::on_chooseTemplateBtn_clicked()
{
    newTemplate = QFileDialog::getOpenFileName(
                this, tr("Choose a template"),QString(), "(*.html *.htm)");
}

void SettingsDialog::on_cleanAfter_currentIndexChanged(int index)
{
    settings->set("slideshow/clean_after",
                  ui.cleanAfter->itemData(index).toString());
}

void SettingsDialog::on_newTabUrl_textChanged(const QString &arg1)
{
    settings->set("main/new_tab_url",arg1);
}

void SettingsDialog::on_resizeRadio_toggled(bool checked)
{
    settings->set("slideshow/resize", checked);
    settings->set("slideshow/auto_scroll", !checked);
    settings->set("slideshow/stretch", !checked);
}

void SettingsDialog::on_scrollRadio_toggled(bool checked)
{
    settings->set("slideshow/resize", !checked);
    settings->set("slideshow/auto_scroll", checked);
    settings->set("slideshow/stretch", !checked);
}

void SettingsDialog::on_noneRadio_toggled(bool checked)
{
    settings->set("slideshow/resize", !checked);
    settings->set("slideshow/auto_scroll", !checked);
    settings->set("slideshow/stretch", !checked);
}

void SettingsDialog::on_stretchRadio_toggled(bool checked)
{
    settings->set("slideshow/resize", !checked);
    settings->set("slideshow/auto_scroll", !checked);
    settings->set("slideshow/stretch",checked);
}

void SettingsDialog::on_playlistUrl_textChanged(const QString &text)
{
    settings->set("slideshow/pages", text);
}

void SettingsDialog::on_tvChk_toggled(bool checked)
{
    settings->set("remote/tv", checked);
}

void SettingsDialog::on_localplaylistChk_toggled(bool checked)
{
    settings->set("remote/localplaylist", checked);
}

void SettingsDialog::on_localfsChk_toggled(bool checked)
{
    settings->set("remote/localfs", checked);
}

void SettingsDialog::on_externalPlayerChk_toggled(bool checked)
{
    settings->set("slideshow/external_player", checked);
}

void SettingsDialog::on_slideshowExternalBrowserChk_toggled(bool checked)
{
    settings->set("slideshow/external_browser", checked);
}

void SettingsDialog::on_listWidget_doubleClicked(const QModelIndex)
{

    creer();
    on_testBtn_clicked();
}
