#ifndef SETTINGSUI_H
#define SETTINGSUI_H

#include <QProcess>
#include <QtNetwork>

#include "settings.h"
#include "treeview.h"

#include "ui_webkioskSettings.h"


class CommandLineParser;
class SettingsDialog : public QDialog
{
    Q_OBJECT

public:
    SettingsDialog(QWidget *parent = NULL,
                   CommandLineParser const* cmdparse = NULL);

public slots:
        void exit();
        void creer();
private:
    Ui::SettingsDialog ui;

    Settings *settings;

    TreeView *m_localPlaylistView;

    QProcess *m_testProcess;

    QString token;
    QString newTemplate;

    bool m_isInit;

    void finalizeUi();
    void applySettings();


    void setText(QLineEdit *obj, const char *path);
    void setChecked(QCheckBox *obj, const char *path, bool value);
    void setSpin(QSpinBox *obj, const char *path);
    void setZoom(QDoubleSpinBox *obj, const char *path);
    void setLinutopTV(bool priv);
    void setPrivatePlaylist(void);
    void setLocalPlaylist(void);
    void radioChoiceInit(void);

    void updateChannelInfo(bool publ);

    bool restrictionsLoadFromFile(QString &filename);
    bool restrictionsSaveToFile(QString &filename);
    void playlistLoadFromFile(QString filename);
    void loadChannels();
    void loadPlaylist();
    void startRemote(bool ddonly);
    QString getSlideBaseUrl(void) const;

private slots:
    void genPlaylistList();
    void genRestrictionsList();
    void on_restrictionsSelection_changed(const QItemSelection &selected,
                          const QItemSelection &deselected);
    void showError(QNetworkReply::NetworkError error);
    void on_helpBtn_clicked();
    void on_testFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void on_testBtn_clicked();
    void on_cancelBtn_clicked();
    void on_okBtn_clicked();

    void on_useBtnGrp_buttonClicked(QAbstractButton *btn);
    void on_fullscreenChk_toggled(bool checked);
    //Ajout de la méthode fixedscreen
    void on_fixedscreenChk_toggled(bool checked);
    void on_resetChk_toggled(bool checked);
    void on_screenSaverChk_toggled(bool checked);
    void on_startupChk_toggled(bool checked);
    void on_passwordChk_toggled(bool checked);
    void on_passwordBtn_clicked();
    void on_fontBtn_clicked();

    void on_hideToolbarChk_toggled(bool checked);
    void on_browseFSChk_toggled(bool checked);
    void on_displayClearChk_toggled(bool checked);
    void on_printChk_toggled(bool checked);
    void on_sessionAtStartupChk_toggled(bool checked);
    void on_homeEdit_textChanged(QString value);

    void on_sessionTimeoutSpin_valueChanged(int value);
    void on_sessionTimeChk_toggled(bool checked);
    void on_sessionTimeSpin_valueChanged(int value);
    void on_postSessionUrlEdit_textEdited(QString value);
    void on_postSessionTimeSpin_valueChanged(int value);
    void on_zoomFactor_valueChanged(double value);
    void on_restrictionsCombo_currentIndexChanged(int value);
    void on_regexChk_toggled(bool checked);
    void on_restrictionAddBtn_clicked();
    void on_restrictionDeleteBtn_clicked();
    void on_restrictionImportBtn_clicked();
    void on_restrictionExportBtn_clicked();

    void on_externalBrowserChk_toggled(bool checked);

    void on_hideMouseChk_toggled(bool checked);
    void on_muteChk_toggled(bool checked);
    void on_displayTimeSpin_valueChanged(int value);
    void on_playlistBtnGrp_buttonClicked(QAbstractButton *btn);
    void on_playlistUsbUpdate_toggled(bool checked);
    void on_remoteEdit_textChanged(QString value);
    void on_refreshPlaylistSpin_valueChanged(int value);
    void on_localPlaylistDeleteBtn_clicked();
    void on_localPlaylistUpBtn_clicked();
    void on_localPlaylistDownBtn_clicked();
    void on_localPlaylistAddBtn_clicked();
    void on_localPlaylistScreenOffBtn_clicked();
    void on_localPlaylistCookiesBtn_clicked();
    void on_usbAuto_toggled(bool value);
    void on_shuffle_toggled(bool checked);
    void on_PlaylistSelectionChanged(const QItemSelection &,
                    const QItemSelection &);

    void on_proxyBtnGrp_buttonClicked(QAbstractButton *btn);
    void on_proxyUserEdit_textChanged(QString s);
    void on_largeur_edit_textChanged(QString s);
    void on_hauteur_edit_textChanged(QString s);
    void on_proxyPasswordEdit_textChanged(QString s);
    void on_proxyUrlEdit_textChanged(QString s);
    void on_invoice_textChanged(QString s);
    void on_mail_textChanged(QString s);
    void on_connect_clicked();
    void on_listWidget_itemClicked(QListWidgetItem *item);
    void on_remoteChk_toggled(bool checked);
    void on_privateServer_toggled(bool checked);
    void on_directChk_toggled(bool checked);
    void on_chooseTemplateBtn_clicked();
    void on_cleanAfter_currentIndexChanged(int index);
    void on_newTabUrl_textChanged(const QString &arg1);
    void on_resizeRadio_toggled(bool checked);
    void on_scrollRadio_toggled(bool checked);
    void on_noneRadio_toggled(bool checked);
    void on_stretchRadio_toggled(bool checked);
    void on_playlistUrl_textChanged(const QString &text);
    void on_privateEdit_textChanged(QString value);

    void on_remotepassword_textChanged(QString s);
    void on_tvChk_toggled(bool);
    void on_localplaylistChk_toggled(bool);
    void on_localfsChk_toggled(bool);
    void on_externalPlayerChk_toggled(bool checked);
    void on_slideshowExternalBrowserChk_toggled(bool checked);
    void on_listWidget_doubleClicked(const QModelIndex);
};

#endif
