#include <QHeaderView>
#include "treeview.h"

TreeView::TreeView(QWidget *parent)
    : QTreeView(parent)
{
    m_model = new QStandardItemModel;
    QStringList headers;
    headers << "URL" << "Duration";
    m_model->setHorizontalHeaderLabels(headers);
    setModel(m_model);

    m_delegate = new SpinBoxDelegate(this);
    setItemDelegateForColumn(1, m_delegate);

    setSelectionMode(QAbstractItemView::MultiSelection);
    // Qt5
    //header()->setSectionResizeMode(QHeaderView::ResizeToContents);
    header()->setSectionResizeMode(QHeaderView::ResizeToContents);

    connect(m_model, SIGNAL(dataChanged(QModelIndex, QModelIndex)),
            this, SLOT(onDataChanged(QModelIndex, QModelIndex)));
}

void TreeView::onDataChanged(QModelIndex a, QModelIndex b)
{
    emit(dataChanged());
}

void TreeView::add(QString url, int duration)
{
    QStandardItem *parentItem = m_model->invisibleRootItem();
    QList<QStandardItem *> tmp;
    tmp << new QStandardItem() << new QStandardItem();
    parentItem->appendRow(tmp);
    int row = m_model->rowCount() - 1;
    QModelIndex idx0 = m_model->index(row, 0, QModelIndex());
    QModelIndex idx1 = m_model->index(row, 1, QModelIndex());
    m_model->setData(idx0, url);
    m_model->setData(idx1, duration);
}

static QList<int> indexesFromList(QModelIndexList &selected, bool reverted = false)
{
    QList<int> indexes;
    QModelIndex idx;
    foreach (idx, selected)
        indexes << idx.row();

    if (reverted)
        qSort(indexes.begin(), indexes.end(), qGreater<int>());

    return indexes;
}

void TreeView::delSelection()
{
    QItemSelectionModel *selModel = selectionModel();
    QModelIndexList selected = selModel->selectedRows();

    QList<int> indexes = indexesFromList(selected, true);
    foreach (int i, indexes)
        m_model->removeRow(i);
}

static bool reverseSort(QModelIndex &a, QModelIndex &b)
{
    return a.row() > b.row();
}

void TreeView::moveSelection(int howMuch)
{
    QList<QStandardItem *> items;

    QItemSelectionModel *selModel = selectionModel();
    QModelIndexList selected = selModel->selectedRows();
    if(howMuch > 0)
        qSort(selected.begin(), selected.end(), reverseSort);
    else
        qSort(selected.begin(), selected.end());

    foreach (QModelIndex idx, selected)
    {
        items << m_model->itemFromIndex(idx);
        int newRow = idx.row() + howMuch;
        if ((newRow < 0) || (newRow >= m_model->rowCount()))
            break;

        QList<QStandardItem *> rowItems = m_model->takeRow(idx.row());
        m_model->insertRow(idx.row() + howMuch, rowItems);
    }

    selModel->clear();
    foreach (QStandardItem *item, items)
        selModel->select(item->index(),
                QItemSelectionModel::Select | QItemSelectionModel::Rows);
}
